"""Module with the classes and functions that manage
the experiments windows and panels.

This file is part of the physbiblio package.
"""
import traceback
from PySide2.QtCore import Qt, QTimer
from PySide2.QtGui import QCursor
from PySide2.QtWidgets import QAction, QLineEdit, QPushButton, QToolTip

try:
    from physbiblio.config import pbConfig
    from physbiblio.database import pBDB
    from physbiblio.view import pBView
    from physbiblio.errors import pBLogger
    from physbiblio.gui.errorManager import pBGUILogger
    from physbiblio.gui.basicDialogs import askYesNo
    from physbiblio.gui.commonClasses import (
        EditObjectWindow,
        PBLabel,
        PBMenu,
        PBTableModel,
        ObjListWindow,
        pBGuiView,
    )
    from physbiblio.gui.catWindows import CatsTreeWindow
    import physbiblio.gui.resourcesPyside2
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())


def editExperiment(parentObject, mainWinObject, editIdExp=None):
    """Open a dialog (`EditExperimentDialog`) to edit an experiment
    and process the output.

    Parameters:
        parentObject: the parent widget
        mainWinObject: the object which has
            the `statusBarMessage` and `setWindowTitle` methods
        editIdCat: the id of the experiment to be edited,
            or `None` to create a new one
    """
    if editIdExp is not None:
        edit = pBDB.exps.getDictByID(editIdExp)
    else:
        edit = None
    newExpWin = EditExperimentDialog(parentObject, experiment=edit)
    newExpWin.exec_()
    if newExpWin.result:
        data = {}
        for k, v in newExpWin.textValues.items():
            s = "%s" % v.text()
            data[k] = s
        if data["name"].strip() != "":
            if "idExp" in data.keys():
                pBLogger.info("Updating experiment %s..." % data["idExp"])
                pBDB.exps.update(data, data["idExp"])
            else:
                pBDB.exps.insert(data)
            message = "Experiment saved"
            mainWinObject.setWindowTitle("PhysBiblio*")
            try:
                parentObject.recreateTable()
            except AttributeError:
                pBLogger.debug(
                    "parentObject has no attribute 'recreateTable'", exc_info=True
                )
        else:
            message = "ERROR: empty experiment name"
    else:
        message = "No modifications to experiments"
    try:
        mainWinObject.statusBarMessage(message)
    except AttributeError:
        pBLogger.debug(
            "mainWinObject has no attribute 'statusBarMessage'", exc_info=True
        )


def deleteExperiment(parentObject, mainWinObject, idExp, name):
    """Ask confirmation and eventually delete the selected experiment

    Parameters:
        parentObject: the parent widget
        mainWinObject: the object which has
            the `statusBarMessage` and `setWindowTitle` methods
        idExp: the id of the experiment to be deleted
        name: the name of the experiment to be deleted
    """
    if askYesNo(
        "Do you really want to delete this experiment "
        + "(ID = '%s', name = '%s')?" % (idExp, name)
    ):
        pBDB.exps.delete(int(idExp))
        mainWinObject.setWindowTitle("PhysBiblio*")
        message = "Experiment deleted"
        try:
            parentObject.recreateTable()
        except AttributeError:
            pBLogger.debug(
                "parentObject has no attribute 'recreateTable'", exc_info=True
            )
    else:
        message = "Nothing changed"
    try:
        mainWinObject.statusBarMessage(message)
    except AttributeError:
        pBLogger.debug(
            "mainWinObject has no attribute 'statusBarMessage'", exc_info=True
        )


class ExpTableModel(PBTableModel):
    """Model for the experiment list"""

    typeClass = "Exps"

    def __init__(self, parent, exp_list, header, askExps=False, previous=[], *args):
        """Extension of `PBTableModel`
        Initialize the model, save the data and the initial selection

        Parameters:
            parent: the parent widget
            exp_list: the list of experiments records from the database
            header: the names of the column fields
            askExps (default False): if True, enable checkboxes for
                selection of experiments
            previous: the list of previously selected experiments
                (default: an empty list)
        """
        self.dataList = exp_list
        super(ExpTableModel, self).__init__(parent, header, askExps, previous, *args)
        self.prepareSelected()

    def getIdentifier(self, element):
        """Get the string that identifies the given element
        in the database

        Parameter:
            element: the database record
        """
        return element["idExp"]

    def data(self, index, role):
        """Return the cell data for the given index and role

        Parameters:
            index: the `QModelIndex` for which the data are required
            role: the desired Qt role for the data

        Output:
            None if the index or the role are not valid,
            the cell content or properties otherwise
        """
        if not index.isValid():
            return None
        row = index.row()
        column = index.column()
        try:
            value = self.dataList[row][self.header[column]]
        except IndexError:
            return None

        if role == Qt.CheckStateRole and self.ask and column == 0:
            if self.selectedElements[self.dataList[row][self.header[0]]] == False:
                return Qt.Unchecked
            else:
                return Qt.Checked
        if role == Qt.EditRole:
            return value
        if role == Qt.DisplayRole:
            return value
        return None

    def setData(self, index, value, role):
        """Set the cell data for the given index and role

        Parameters:
            index: the `QModelIndex` for which the data are required
            value: the new data value
            role: the desired Qt role for the data

        Output:
            True if correctly completed,
            False if the `index` is not valid
        """
        if not index.isValid():
            return False
        if role == Qt.CheckStateRole and index.column() == 0:
            if value == Qt.Checked:
                self.selectedElements[self.dataList[index.row()]["idExp"]] = True
            else:
                self.selectedElements[self.dataList[index.row()]["idExp"]] = False

        self.dataChanged.emit(index, index)
        return True


class ExpsListWindow(ObjListWindow):
    """create a window for printing the list of experiments"""

    colcnt = len(pBDB.tableCols["experiments"])
    colContents = [
        pBDB.tableCols["experiments"][j]
        for j in range(len(pBDB.tableCols["experiments"]))
    ]

    def __init__(
        self, parent=None, askExps=False, askForBib=None, askForCat=None, previous=[]
    ):
        """Constructor, extends `ObjListWindow.__init__`
        with more settings and parameters

        Parameters:
            parent: the parent widget
            askExps: if True, checkboxes will be used
            askForBib: the key identifying the bibtex entry for which
                the experiments are asked
            askForCat: the ID identifying the category for which
                the experiments are asked
            previous: the list (default empty) of initially selected
                experiments
        """
        self.askExps = askExps
        self.askForBib = askForBib
        self.askForCat = askForCat
        self.previous = previous
        self.marked = []
        self.result = False
        self.exps = []
        self.newExpButton = None
        self.acceptButton = None
        self.cancelButton = None
        self.menu = None
        self.timer = None

        ObjListWindow.__init__(self, parent)
        self.setWindowTitle("List of experiments")

        self.createTable()

    def populateAskExp(self):
        """If selection of experiments is allowed, add some information
        on the bibtex/category for which the experiments are requested
        and a simple message, then create few required empty lists
        """
        if self.askExps:
            if self.askForBib is not None:
                try:
                    bibitem = pBDB.bibs.getByBibkey(self.askForBib, saveQuery=False)[0]
                except IndexError:
                    pBGUILogger.warning(
                        "The entry '%s' " % self.askForBib + "is not in the database!",
                        exc_info=True,
                    )
                    return
                try:
                    if bibitem["inspire"] != "" and bibitem["inspire"] is not None:
                        link = "<a href='%s'>%s</a>" % (
                            pBView.getLink(self.askForBib, "inspire"),
                            self.askForBib,
                        )
                    elif bibitem["arxiv"] != "" and bibitem["arxiv"] is not None:
                        link = "<a href='%s'>%s</a>" % (
                            pBView.getLink(self.askForBib, "arxiv"),
                            self.askForBib,
                        )
                    elif bibitem["doi"] != "" and bibitem["doi"] is not None:
                        link = "<a href='%s'>%s</a>" % (
                            pBView.getLink(self.askForBib, "doi"),
                            self.askForBib,
                        )
                    else:
                        link = self.askForBib
                    bibtext = PBLabel(
                        "Mark experiments for the following entry:<br>"
                        + "<b>key</b>:<br>%s<br>" % link
                        + "<b>author(s)</b>:<br>%s<br>" % bibitem["author"]
                        + "<b>title</b>:<br>%s<br>" % bibitem["title"]
                    )
                except KeyError:
                    bibtext = PBLabel(
                        "Mark experiments for the following entry:<br>"
                        + "<b>key</b>:<br>%s<br>" % (self.askForBib)
                    )
                self.currLayout.addWidget(bibtext)
            elif self.askForCat is not None:
                raise NotImplementedError("This feature is not implemented")
            else:
                self.currLayout.addWidget(PBLabel("Select the desired experiments:"))
            self.marked = []
            self.parent().selectedExps = []
        return True

    def onCancel(self):
        """Reject the dialog content and close the window"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the dialog content
        (update the list of selected experiments)
        and close the window.
        """
        self.parent().selectedExps = [
            idE
            for idE in self.tableModel.selectedElements.keys()
            if self.tableModel.selectedElements[idE] == True
        ]
        self.result = "Ok"
        self.close()

    def onNewExp(self):
        """Action to perform when the creation
        of a new experiment is requested
        """
        editExperiment(self, self.parent())

    def keyPressEvent(self, e):
        """Manage the key press events.
        Do nothing unless `Esc` is pressed:
        in this case close the dialog
        """
        if e.key() == Qt.Key_Escape:
            self.close()

    def createTable(self):
        """Create the dialog content, connect the model to the view
        and eventually add the buttons at the end
        """
        self.populateAskExp()

        self.exps = pBDB.exps.getAll()

        self.tableModel = ExpTableModel(
            self,
            self.exps,
            pBDB.tableCols["experiments"],
            askExps=self.askExps,
            previous=self.previous,
        )
        self.addFilterInput("Filter experiment")
        self.setProxyStuff(1, Qt.AscendingOrder)

        self.finalizeTable()

        self.newExpButton = QPushButton("Add new experiment", self)
        self.newExpButton.clicked.connect(self.onNewExp)
        self.currLayout.addWidget(self.newExpButton)

        if self.askExps:
            self.acceptButton = QPushButton("OK", self)
            self.acceptButton.clicked.connect(self.onOk)
            self.currLayout.addWidget(self.acceptButton)

            # cancel button
            self.cancelButton = QPushButton("Cancel", self)
            self.cancelButton.clicked.connect(self.onCancel)
            self.cancelButton.setAutoDefault(True)
            self.currLayout.addWidget(self.cancelButton)

    def triggeredContextMenuEvent(self, row, col, event):
        """Process event when mouse right-clicks an item.
        Opens a menu with a number of actions

        Parameter:
            row: the row number
            col: a the column number
            event: a `QEvent` instance, used to obtain the mouse
                position where to open the menu
        """
        index = self.proxyModel.index(row, col)
        if not index.isValid():
            return
        try:
            idExp = str(self.proxyModel.sibling(row, 0, index).data())
            expName = str(self.proxyModel.sibling(row, 1, index).data())
        except AttributeError:
            return

        menu = PBMenu()
        self.menu = menu
        titAction = QAction("--Experiment: %s--" % expName)
        titAction.setDisabled(True)
        bibAction = QAction("Open list of corresponding entries")
        modAction = QAction("Modify")
        delAction = QAction("Delete")
        catAction = menu.addAction("Categories")
        menu.possibleActions = [
            titAction,
            None,
            bibAction,
            None,
            modAction,
            delAction,
            None,
            catAction,
        ]
        menu.fillMenu()

        action = menu.exec_(event.globalPos())

        if action == bibAction:
            self.parent().reloadMainContent(pBDB.bibs.getByExp(idExp))
        elif action == modAction:
            editExperiment(self, self.parent(), idExp)
        elif action == delAction:
            deleteExperiment(self, self.parent(), idExp, expName)
        elif action == catAction:
            previous = [a["idCat"] for a in pBDB.cats.getByExp(idExp)]
            selectCats = CatsTreeWindow(
                parent=self.parent(),
                askCats=True,
                askForExp=idExp,
                expButton=False,
                previous=previous,
            )
            selectCats.exec_()
            if selectCats.result == "Ok":
                cats = self.parent().selectedCats
                for p in previous:
                    if p not in cats:
                        pBDB.catExp.delete(p, idExp)
                for c in cats:
                    if c not in previous:
                        pBDB.catExp.insert(c, idExp)
                self.parent().statusBarMessage(
                    "Categories for '%s' successfully inserted" % expName
                )
        return True

    def handleItemEntered(self, index):
        """Process event when mouse enters an item and
        create a `QTooltip` which describes the category, with a timer

        Parameter:
            index: a `QModelIndex` instance
        """
        if index.isValid():
            row = index.row()
        else:
            return
        idExp = str(self.proxyModel.sibling(row, 0, index).data())
        try:
            self.timer.stop()
            QToolTip.showText(QCursor.pos(), "", self.tableview.viewport())
        except AttributeError:
            pass
        try:
            expData = pBDB.exps.getByID(idExp)[0]
        except IndexError:
            pBGUILogger.exception("Failed in finding experiment")
            return
        self.timer = QTimer(self)
        self.timer.setSingleShot(True)
        self.timer.timeout.connect(
            lambda: QToolTip.showText(
                QCursor.pos(),
                "{idE}: {exp}\nCorresponding entries: {en}\n".format(
                    idE=idExp, exp=expData["name"], en=pBDB.bibExp.countByExp(idExp)
                )
                + "Associated categories: {ca}".format(
                    ca=pBDB.catExp.countByExp(idExp)
                ),
                self.tableview.viewport(),
                self.tableview.visualRect(index),
                3000,
            )
        )
        self.timer.start(500)

    def cellClick(self, index):
        """Process event when mouse clicks an item.
        Currently not used

        Parameter:
            index: a `QModelIndex` instance
        """
        if index.isValid():
            row = index.row()
        else:
            return
        idExp = str(self.proxyModel.sibling(row, 0, index).data())
        return True

    def cellDoubleClick(self, index):
        """Process event when mouse double clicks an item.
        Opens a link if some columns

        Parameter:
            index: a `QModelIndex` instance
        """
        if index.isValid():
            row = index.row()
            col = index.column()
        else:
            return
        idExp = str(self.proxyModel.sibling(row, 0, index).data())
        if self.colContents[col] == "inspire" or self.colContents[col] == "homepage":
            link = self.proxyModel.sibling(row, col, index).data()
            if link == "" or link is None:
                self.parent().reloadMainContent(pBDB.bibs.getByExp(idExp))
                return
            if self.colContents[col] == "inspire":
                link = pbConfig.inspireRecord + link
            pBLogger.debug("Opening '%s'..." % link)
            try:
                pBGuiView.openLink(link, "link")
            except Exception:
                pBLogger.warning("Opening link '%s' failed!" % link, exc_info=True)
        else:
            self.parent().reloadMainContent(pBDB.bibs.getByExp(idExp))
        return True


class EditExperimentDialog(EditObjectWindow):
    """create a window for editing or creating an experiment"""

    def __init__(self, parent=None, experiment=None):
        """Extend `EditObjectWindow.__init__` to define self.data
        and call createForm

        Parameters:
            parent: the parent widget
            experiment: the database record or dictionary which
                contains the experiment information
        """
        super(EditExperimentDialog, self).__init__(parent)
        if experiment is None:
            self.data = {}
            for k in pBDB.tableCols["experiments"]:
                self.data[k] = ""
        else:
            self.data = experiment
        self.acceptButton = None
        self.cancelButton = None
        self.createForm()

    def createForm(self):
        """Create the form labels, fields, buttons"""
        self.setWindowTitle("Edit experiment")

        i = 0
        for k in pBDB.tableCols["experiments"]:
            val = self.data[k]
            if k != "idExp" or (k == "idExp" and self.data[k] != ""):
                i += 1
                self.currGrid.addWidget(PBLabel(k), i * 2 - 1, 0)
                self.currGrid.addWidget(
                    PBLabel("(%s)" % pBDB.descriptions["experiments"][k]), i * 2 - 1, 1
                )
                self.textValues[k] = QLineEdit(str(val))
                if k == "idExp":
                    self.textValues[k].setEnabled(False)
                self.currGrid.addWidget(self.textValues[k], i * 2, 0, 1, 2)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        self.currGrid.addWidget(self.acceptButton, i * 2 + 1, 0)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.currGrid.addWidget(self.cancelButton, i * 2 + 1, 1)

        self.setGeometry(100, 100, 400, 50 * i)
        self.centerWindow()



#!/usr/bin/env python
"""Test file for the physbiblio.errors module.

This file is part of the physbiblio package.
"""
import sys
import traceback

if sys.version_info[0] < 3:
    import unittest2 as unittest
    from mock import patch
    from StringIO import StringIO
else:
    import unittest
    from unittest.mock import patch
    from io import StringIO

try:
    from physbiblio.setuptests import *
    from physbiblio.errors import PBErrorManagerClass
    from physbiblio.config import pbConfig
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


class TestErrors(unittest.TestCase):
    """Test PBErrorManagerClass stuff"""

    @classmethod
    def setUpClass(self):
        """settings"""
        with patch.dict(
            pbConfig.params,
            {"loggingLevel": 3, "logFileName": logFileName},
            clear=False,
        ):
            self.pBErrorManager = PBErrorManagerClass("testlogger")

    def reset(self, m):
        """Clear a StringIO object"""
        m.seek(0)
        m.truncate(0)

    def resetLogFile(self):
        """Reset the logfile content"""
        open(logFileName, "w").close()

    def readLogFile(self):
        """Read the logfile content"""
        with open(logFileName) as logFile:
            log_new = logFile.read()
        return log_new

    def test_critical(self):
        """Test pBLogger.critical with and without exception"""
        self.resetLogFile()
        try:
            raise Exception("Fake error")
        except Exception as e:
            self.pBErrorManager.logger.critical(str(e))
        log_new = self.readLogFile()
        self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), log_new)
        self.assertIn("CRITICAL : [test_errors.test_critical] Fake error", log_new)
        self.assertNotIn("Traceback (most recent call last):", log_new)
        try:
            raise Exception("Fake error")
        except Exception as e:
            self.pBErrorManager.logger.critical(str(e), exc_info=True)
        log_new = self.readLogFile()
        self.assertIn("Traceback (most recent call last):", log_new)

    def test_exception(self):
        """Test pBLogger.exception"""
        self.resetLogFile()
        try:
            raise Exception("Fake error")
        except Exception as e:
            self.pBErrorManager.logger.exception(str(e))
        try:
            raise Exception("New fake error")
        except Exception as e:
            self.pBErrorManager.logger.error(str(e))
        log_new = self.readLogFile()
        self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), log_new)
        self.assertIn("ERROR : [test_errors.test_exception] Fake error", log_new)
        self.assertIn("ERROR : [test_errors.test_exception] New fake error", log_new)
        self.assertIn("Traceback (most recent call last):", log_new)

    def test_warning(self):
        """Test pBLogger.warning"""
        self.resetLogFile()
        try:
            raise Exception("Fake warning")
        except Exception as e:
            self.pBErrorManager.logger.warning(str(e))
        log_new = self.readLogFile()
        self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), log_new)
        self.assertIn("WARNING : [test_errors.test_warning] Fake warning", log_new)

    def test_info(self):
        """Test pBLogger.info"""
        self.resetLogFile()
        self.pBErrorManager.logger.info("Some info")
        log_new = self.readLogFile()
        self.assertIn(datetime.datetime.now().strftime("%Y-%m-%d %H:%M"), log_new)
        self.assertIn("INFO : [test_errors.test_info] Some info", log_new)

    def test_tempHandler(self):
        """Test stuff as the temporary handler"""
        self.assertEqual(self.pBErrorManager.tempsh, [])
        self.assertEqual(len(self.pBErrorManager.logger.handlers), 2)
        with patch("sys.stdout", new_callable=StringIO) as _mock:
            self.assertTrue(
                self.pBErrorManager.tempHandler(sys.stdout, format="%(message)s")
            )
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 3)
            self.assertEqual(len(self.pBErrorManager.tempsh), 1)
            self.reset(_mock)
            self.pBErrorManager.logger.critical("Fake critical")
            self.assertEqual(_mock.getvalue(), "Fake critical\n")
            self.pBErrorManager.rmTempHandler()
            self.assertEqual(self.pBErrorManager.tempsh, [])
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 2)
            self.assertEqual(len(self.pBErrorManager.tempsh), 0)
            self.reset(_mock)
            self.assertEqual(_mock.getvalue(), "")
            self.pBErrorManager.logger.critical("Fake critical")
            self.assertEqual(_mock.getvalue(), "")
            self.assertTrue(
                self.pBErrorManager.tempHandler(sys.stdout, format="%(message)s")
            )
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 3)
            self.assertTrue(
                self.pBErrorManager.tempHandler(sys.stdout, format="%(message)s")
            )
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 4)
            self.assertTrue(
                self.pBErrorManager.tempHandler(sys.stdout, format="%(message)s")
            )
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 5)
            self.assertEqual(len(self.pBErrorManager.tempsh), 3)
            self.pBErrorManager.rmTempHandler()
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 4)
            self.assertEqual(len(self.pBErrorManager.tempsh), 2)
            self.pBErrorManager.rmAllTempHandlers()
            self.assertEqual(len(self.pBErrorManager.logger.handlers), 2)
            self.assertEqual(len(self.pBErrorManager.tempsh), 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""Test file for the physbiblio.view module.

This file is part of the physbiblio package.
"""
import sys
import traceback

if sys.version_info[0] < 3:
    import unittest2 as unittest
    from mock import call, patch
else:
    import unittest
    from unittest.mock import call, patch

try:
    from physbiblio.setuptests import *
    from physbiblio.config import pbConfig
    from physbiblio.database import pBDB
    from physbiblio.view import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


@unittest.skipIf(skipTestsSettings.long, "Long tests")
class TestViewMethods(unittest.TestCase):
    """Tests for methods in physbiblio.view"""

    def test_init(self):
        """test instance properties"""
        tmp = ViewEntry()
        self.assertIsInstance(tmp, ViewEntry)
        self.assertEqual(tmp.webApp, pbConfig.params["webApplication"])
        self.assertEqual(tmp.inspireRecord, pbConfig.inspireRecord)
        self.assertEqual(tmp.inspireSearch, pbConfig.inspireSearchBase + "?p=find+")
        with patch.dict(pbConfig.params, {"webApplication": "someapp"}, clear=False):
            tmp = ViewEntry()
            self.assertEqual(tmp.webApp, "someapp")

    def test_getLink(self):
        """Test getLink function with different inputs"""
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=["abc...123", "1507.08204", "", "", "abc...123"],
        ) as _mock:
            self.assertEqual(
                pBView.getLink("a", "ads"), "%sabc...123" % pbConfig.adsUrl
            )
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=["", "1507.08204", "", "", "1507.08204"],
        ) as _mock:
            self.assertEqual(
                pBView.getLink("a", "arxiv"), "%s/abs/1507.08204" % pbConfig.arxivUrl
            )
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=[
                "abc...123",
                "1234.56789",
                "10.1088/0954-3899/43/3/033001",
                "123456789",
                "10.1088/0954-3899/43/3/033001",
            ],
        ) as _mock:
            self.assertEqual(
                pBView.getLink("a", "doi"),
                "%s10.1088/0954-3899/43/3/033001" % pbConfig.doiUrl,
            )
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=[
                "abc...123",
                "1234.56789",
                "10.1088/0954-3899/43/3/033001",
                "1385583",
                "1385583",
            ],
        ) as _mock:
            self.assertEqual(
                pBView.getLink("a", "inspire"), "https://inspirehep.net/record/1385583"
            )
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=[
                "abc...123",
                "1507.08204",
                "10.1088/0954-3899/43/3/033001",
                "",
            ],
        ) as _mock:
            self.assertEqual(
                pBView.getLink("a", "inspire"),
                "https://inspirehep.net/search?p=find+1507.08204",
            )
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=["abc...123", "", "10.1088/0954-3899/43/3/033001", False],
        ) as _mock:
            self.assertEqual(
                pBView.getLink("a", "inspire"), "https://inspirehep.net/search?p=find+a"
            )
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=["", "", "", ""],
        ) as _mock:
            self.assertFalse(pBView.getLink("a", "arxiv"))
        with patch(
            "physbiblio.database.Entries.getField",
            autospec=True,
            side_effect=["", "", "", ""],
        ) as _mock:
            self.assertFalse(pBView.getLink("a", "arxiv"))

    def test_openLink(self):
        """Test openLink"""
        origwebapp = pBView.webApp
        with patch(
            "physbiblio.view.ViewEntry.getLink", autospec=True, return_value="test_link"
        ) as _gl, patch("logging.Logger.info") as _i, patch(
            "logging.Logger.warning"
        ) as _w, patch(
            "subprocess.Popen", autospec=USE_AUTOSPEC_CLASS
        ) as _po:
            pBView.openLink(["a", "b"], arg="file")
            _gl.assert_has_calls(
                [
                    call(pBView, "a", arg="file", fileArg=None),
                    call(pBView, "b", arg="file", fileArg=None),
                ]
            )
            _po.assert_not_called()
            _gl.reset_mock()
            pBView.webApp = "test"
            pBView.openLink("a")
            _gl.assert_called_once_with(pBView, "a", arg="arxiv", fileArg=None)
            _po.assert_called_once_with(
                [pBView.webApp, "test_link"],
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
            )
            _i.assert_called_once_with("Opening 'test_link'...")
            _w.assert_not_called()
            _i.reset_mock()
            pBView.webApp = ""
            pBView.openLink("a")
            _i.assert_not_called()
            _w.assert_not_called()
            _gl.reset_mock()
            _po.reset_mock()
            pBView.webApp = "test"
            pBView.openLink("a", arg="link")
            _po.assert_called_once_with(
                [pBView.webApp, "a"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT
            )
            _i.assert_called_once_with("Opening 'a'...")
            _po.side_effect = OSError
            pBView.openLink("a", arg="link")
            _w.assert_called_once_with("Opening link for 'a' failed!")
            _w.reset_mock()
            _gl.reset_mock()
            _gl.return_value = ""
            pBView.openLink("a", arg="doi", fileArg="arg")
            _gl.assert_called_once_with(pBView, "a", arg="doi", fileArg="arg")
            _w.assert_called_once_with("Impossible to get the 'doi' link for entry 'a'")
        pBView.webApp = origwebapp


if __name__ == "__main__":
    unittest.main()



"""Module that contains the PBBibTexWriter class
(needed to override _entry_to_bibtex).

This file is part of the physbiblio package.
"""
from bibtexparser.bwriter import BibTexWriter


class PBBibTexWriter(BibTexWriter):
    """This class is used to override _entry_to_bibtex"""

    def __init__(self):
        """Constructor for the PBBibTexWriter class.
        Uses parent class constructor
        and adds some additional properties.
        """
        super(PBBibTexWriter, self).__init__()
        # use 13 characters for the field name:
        self._max_field_width = 13
        # order of fields in output
        self.display_order = [
            "author",
            "collaboration",
            "title",
            "booktitle",
            "publisher",
            "journal",
            "volume",
            "year",
            "pages",
            "russian",
            "archiveprefix",
            "primaryclass",
            "eprint",
            "doi",
            "reportNumber",
        ]
        self.bracket_fields = [
            "title",
            "booktitle",
            "www",
            "note",
            "abstract",
            "comment",
            "article",
            "url",
        ]
        self.excluded_fields = ["adsnote", "adsurl", "slaccitation"]
        # Necessary to avoid a change of the ordering of the bibtex entries:
        self.order_entries_by = None
        self.comma_first = False

    def _entry_to_bibtex(self, entry):
        """Redefine the function that writes
        the entries in the bib file.
        Adapted from
        `bibtexparser.bwriter.BibTexWriter._entry_to_bibtex`.

        Parameters:
            entry: a dictionary generated by bibtexparser.

        Output:
            the bibtex string
        """
        bibtex = ""
        # Write BibTeX key
        bibtex += (
            "@"
            + entry["ENTRYTYPE"].capitalize()
            + "{"
            + (entry["ID"] if entry["ID"] is not None else "")
        )

        # create display_order of fields for this entry
        # first those keys which are both
        # in self.display_order and in entry.keys
        display_order = [i for i in self.display_order if i in entry]
        # then all the other fields sorted alphabetically
        display_order += [
            i
            for i in sorted(entry)
            if i not in self.display_order and i not in self.excluded_fields
        ]

        # Write field = value lines
        for field in [i for i in display_order if i not in ["ENTRYTYPE", "ID"]]:
            try:
                entry[field] = entry[field].replace("{{{", "{").replace("}}}", "}")
                bibtex += (
                    ",\n"
                    + self.indent
                    + "{0:>{1}}".format(field, self._max_field_width)
                    + ' = "'
                    + (
                        "{" + entry[field] + "}"
                        if field in self.bracket_fields
                        and (entry[field][0] != "{" or entry[field][-1] != "}")
                        else entry[field]
                    )
                    + '"'
                )
            except TypeError:
                raise TypeError(
                    u"The field %s in entry %s must be a string" % (field, entry["ID"])
                )
        bibtex += ",\n}\n" + self.entry_separator
        return bibtex


pbWriter = PBBibTexWriter()



"""Module that deals with importing info from the ADS API.

This file is part of the physbiblio package.
"""
import traceback
import ads

try:
    from physbiblio.config import pbConfig
    from physbiblio.errors import pBLogger
    from physbiblio.webimport.webInterf import WebInterf
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise


class WebSearch(WebInterf):
    """Subclass of WebInterf that can connect to ADS to perform searches"""

    def __init__(self):
        """Initialize the object and define some basic properties"""
        WebInterf.__init__(self)
        self.name = "ADS-NASA fetcher"
        self.url = "https://ui.adsabs.harvard.edu/"
        self.loadFields = [
            "abstract",
            "arxiv_class",
            "author",
            "bibcode",
            "citation_count",
            "doi",
            "first_author",
            "pubdate",
            "title",
            "year",
        ]
        self.fewFields = ["author", "first_author", "bibcode", "id", "year", "title"]

    def getGenericInfo(
        self, string, fields, rows=pbConfig.params["maxExternalAPIResults"]
    ):
        """Use the unofficial python client for the ADS API to obtain
        a list of results from a given search string

        Parameters:
            string: the search string
            fields: a list with the names of the required fields
            rows: the number of rows to obtain

        Output:
            a list of ads objects with the obtained entries
        """
        ads.config.token = pbConfig.params["ADSToken"]
        try:
            self.q = ads.SearchQuery(q=string, fl=fields, rows=rows)
            l = list(self.q)
        except ads.exceptions.APIResponseError:
            pBLogger.exception("Unauthorized use of ADS API. Is your token valid?")
        except Exception:
            pBLogger.exception("Something went wrong while fetching ADS", exc_info=True)
        else:
            pBLogger.info(self.getLimitInfo())
            return l
        return []

    def getBibtexs(self, bibcodes):
        """Obtain a string containing the bibtex entries for all the
        requested bibcodes

        Parameter:
            bibcodes: a single bibcode
                (string containing the ADS identifier of a given entry)
                or a list of bibcodes

        Output:
            a string with all the bibtex entries
        """
        ads.config.token = pbConfig.params["ADSToken"]
        try:
            self.q = ads.ExportQuery(bibcodes=bibcodes, format="bibtex")
            export = self.q.execute()
        except ads.exceptions.APIResponseError:
            pBLogger.exception("Unauthorized use of ADS API. Is your token valid?")
        except Exception:
            pBLogger.exception(
                "Something went wrong while exporting ADS query", exc_info=True
            )
        else:
            pBLogger.info(self.getLimitInfo())
            return export
        return ""

    def retrieveUrlFirst(self, string):
        """Retrieves the first result from the given search.

        Parameters:
            string: the search string

        Output:
            returns a bibtex string with the first obtained bibtex entry
        """
        a = self.getGenericInfo(string, self.fewFields)
        if len(a) > 0:
            return self.getBibtexs(a[0].bibcode)
        else:
            return ""

    def retrieveUrlAll(self, string):
        """Retrieves all the results from the given search.

        Parameters:
            string: the search string

        Output:
            returns a string with all the bibtex entries
        """
        a = self.getGenericInfo(string, self.fewFields)
        return self.getBibtexs([p.bibcode for p in a])

    def getField(self, string, field):
        """Perform a search and return the value of a specific field
        as given by the ADS API

        Parameters:
            string: the search string
            field: the field name (must be in self.loadFields)

        Output:
            a list with the field values for all the obtained entries
        """
        if not field in self.loadFields:
            pBLogger.warning("Invalid requested field in ADS query: %s" % field)
            return []
        a = self.getGenericInfo(string, self.loadFields)
        return [getattr(p, field) for p in a]

    def getLimitInfo(self):
        """Return a dictionary with the information on the query limits"""
        lims = self.q.response.get_ratelimits()
        return "Remaining queries: %s/%s" % (lims["remaining"], lims["limit"])



"""Module with the classes and functions
that manage the some dialog windows.

This file is part of the physbiblio package.
"""
import traceback
import ast
import six
from PySide2.QtCore import Qt, Signal
from PySide2.QtGui import QTextCursor
from PySide2.QtWidgets import (
    QCheckBox,
    QComboBox,
    QDesktopWidget,
    QGridLayout,
    QLineEdit,
    QPlainTextEdit,
    QProgressBar,
    QPushButton,
    QTableWidgetItem,
    QTextEdit,
    QVBoxLayout,
)

try:
    from physbiblio.config import pbConfig, configuration_params
    from physbiblio.errors import pBLogger
    from physbiblio.database import pBDB
    from physbiblio.webimport.webInterf import physBiblioWeb
    from physbiblio.gui.errorManager import pBGUILogger
    from physbiblio.gui.basicDialogs import (
        askDirName,
        askFileNames,
        askSaveFileName,
        askYesNo,
        infoMessage,
    )
    from physbiblio.gui.commonClasses import (
        PBComboBox,
        PBDDTableWidget,
        PBDialog,
        PBImportedTableModel,
        PBLabel,
        PBLabelRight,
        PBTableView,
        PBTrueFalseCombo,
        ObjListWindow,
    )
    from physbiblio.gui.catWindows import CatsTreeWindow
    import physbiblio.gui.resourcesPyside2
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())


class ConfigEditColumns(PBDialog):
    """Extend `PBDialog` to ask the columns
    that must appear in the main table
    """

    excludeCols = [
        "crossref",
        "bibtex",
        "exp_paper",
        "lecture",
        "phd_thesis",
        "review",
        "proceeding",
        "book",
        "noUpdate",
        "bibdict",
        "abstract",
    ]
    moreCols = [
        "title",
        "author",
        "journal",
        "volume",
        "pages",
        "primaryclass",
        "booktitle",
        "reportnumber",
    ]

    def __init__(self, parent=None, previous=None):
        """Extend `PBDialog.__init__` and create the form structure

        Parameters:
            parent: teh parent widget
            previous: list of columns which must appear
                as selected at the beginning
        """
        super(ConfigEditColumns, self).__init__(parent)
        self.gridlayout = None
        self.items = []
        self.listAll = None
        self.listSel = None
        self.allItems = None
        self.selItems = None
        self.previousSelected = (
            previous if previous is not None else pbConfig.params["bibtexListColumns"]
        )
        self.selected = self.previousSelected
        self.result = False
        self.acceptButton = None
        self.cancelButton = None
        self.initUI()

    def onCancel(self):
        """Reject the output (set self.result to False and close)"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the output and prepare `self.selected`"""
        self.result = True
        self.selected = []
        for row in range(self.listSel.rowCount()):
            self.selected.append(self.listSel.item(row, 0).text())
        self.close()

    def initUI(self):
        """Initialize the `PBDDTableWidget`s and their content,
        plus the buttons and labels
        """
        self.gridlayout = QGridLayout()
        self.setLayout(self.gridlayout)

        self.items = []
        self.listAll = PBDDTableWidget(self, "Available columns")
        self.listSel = PBDDTableWidget(self, "Selected columns")
        self.gridlayout.addWidget(
            PBLabel("Drag and drop items to order visible columns"), 0, 0, 1, 2
        )
        self.gridlayout.addWidget(self.listAll, 1, 0)
        self.gridlayout.addWidget(self.listSel, 1, 1)

        self.allItems = list(pBDB.descriptions["entries"]) + self.moreCols
        self.selItems = self.previousSelected
        isel = 0
        iall = 0
        for col in self.selItems + [i for i in self.allItems if i not in self.selItems]:
            if col in self.excludeCols:
                continue
            item = QTableWidgetItem(col)
            self.items.append(item)
            if col in self.selItems:
                self.listSel.insertRow(isel)
                self.listSel.setItem(isel, 0, item)
                isel += 1
            else:
                self.listAll.insertRow(iall)
                self.listAll.setItem(iall, 0, item)
                iall += 1

        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        self.gridlayout.addWidget(self.acceptButton, 2, 0)

        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.gridlayout.addWidget(self.cancelButton, 2, 1)


class ConfigWindow(PBDialog):
    """Create a window for editing the configuration settings"""

    def __init__(self, parent=None):
        """Simple extension of `PBDialog.__init__`"""
        super(ConfigWindow, self).__init__(parent)
        self.textValues = []
        self.result = False
        self.selectedCats = None
        self.selectedExps = None
        self.grid = None
        self.acceptButton = None
        self.cancelButton = None
        self.initUI()

    def onCancel(self):
        """Reject the output (set self.result to False and close)"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the output (set self.result to True and close)"""
        self.result = True
        self.close()

    def editPDFFolder(self):
        """Open a dialog to select a new folder name
        for the PDF path, and save the result
        in the `ConfigWindow` interface
        """
        ix = pbConfig.paramOrder.index("pdfFolder")
        folder = askDirName(
            parent=None,
            dir=self.textValues[ix][1].text(),
            title="Directory for saving PDF files:",
        )
        if folder.strip() != "":
            self.textValues[ix][1].setText(str(folder))

    def editFile(
        self, paramkey="logFileName", text="Name for the log file", filter="*.log"
    ):
        """Open a dialog to select a new file name
        for a configuration parameter, and save the result
        in the `ConfigWindow` interface

        Parameters:
            paramkey: the parameter name in the configuration dictionary
            text: description in the file dialog
            filter: filter the folder content in the file dialog
        """
        if paramkey not in pbConfig.paramOrder:
            pBLogger.warning("Invalid paramkey: '%s'" % paramkey)
            return
        ix = pbConfig.paramOrder.index(paramkey)
        fname = askSaveFileName(
            parent=None, title=text, dir=self.textValues[ix][1].text(), filter=filter
        )
        if fname.strip() != "":
            self.textValues[ix][1].setText(str(fname))

    def editColumns(self):
        """Open a dialog to select and/or reorder the list of columns
        to show in the entries list, and save the result
        in the `ConfigWindow` interface
        """
        ix = pbConfig.paramOrder.index("bibtexListColumns")
        window = ConfigEditColumns(
            self, ast.literal_eval(self.textValues[ix][1].text().strip())
        )
        window.exec_()
        if window.result:
            columns = window.selected
            self.textValues[ix][1].setText(str(columns))

    def editDefCats(self):
        """Open a dialog to select a the default categories
        for the imported entries, and save the result
        in the `ConfigWindow` interface
        """
        ix = pbConfig.paramOrder.index("defaultCategories")
        selectCats = CatsTreeWindow(
            parent=self,
            askCats=True,
            expButton=False,
            previous=ast.literal_eval(self.textValues[ix][1].text().strip()),
        )
        selectCats.exec_()
        if selectCats.result == "Ok":
            self.textValues[ix][1].setText(str(self.selectedCats))

    def initUI(self):
        """Create and fill the `QGridLayout`"""
        self.setWindowTitle("Configuration")

        grid = QGridLayout()
        self.grid = grid
        grid.setSpacing(1)

        i = 0
        for k in pbConfig.paramOrder:
            i += 1
            val = (
                pbConfig.params[k]
                if isinstance(pbConfig.params[k], six.string_types)
                else str(pbConfig.params[k])
            )
            grid.addWidget(
                PBLabel(
                    "%s (<i>%s</i>%s)"
                    % (
                        configuration_params[k].description,
                        k,
                        " - global setting" if configuration_params[k].isGlobal else "",
                    )
                ),
                i - 1,
                0,
                1,
                2,
            )
            if k == "bibtexListColumns":
                self.textValues.append([k, QPushButton(val)])
                self.textValues[-1][1].clicked.connect(self.editColumns)
            elif k == "pdfFolder":
                self.textValues.append([k, QPushButton(val)])
                self.textValues[-1][1].clicked.connect(self.editPDFFolder)
            elif k == "loggingLevel":
                try:
                    self.textValues.append(
                        [
                            k,
                            PBComboBox(
                                self,
                                pbConfig.loggingLevels,
                                pbConfig.loggingLevels[int(val)],
                            ),
                        ]
                    )
                except (IndexError, ValueError):
                    pBGUILogger.warning(
                        "Invalid string for 'loggingLevel' param. Reset to default"
                    )
                    self.textValues.append(
                        [
                            k,
                            PBComboBox(
                                self,
                                pbConfig.loggingLevels,
                                pbConfig.loggingLevels[
                                    int(configuration_params["loggingLevel"].default)
                                ],
                            ),
                        ]
                    )
            elif k == "logFileName":
                self.textValues.append([k, QPushButton(val)])
                self.textValues[-1][1].clicked.connect(self.editFile)
            elif k == "defaultCategories":
                self.textValues.append([k, QPushButton(val)])
                self.textValues[-1][1].clicked.connect(self.editDefCats)
            elif configuration_params[k].special == "boolean":
                self.textValues.append([k, PBTrueFalseCombo(self, val)])
            else:
                self.textValues.append([k, QLineEdit(val)])
            grid.addWidget(self.textValues[i - 1][1], i - 1, 2, 1, 2)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        # width = self.acceptButton.fontMetrics().boundingRect('OK').width() + 7
        # self.acceptButton.setMaximumWidth(width)
        grid.addWidget(self.acceptButton, i, 0)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        # width = self.cancelButton.fontMetrics().boundingRect('Cancel').width() + 7
        # self.cancelButton.setMaximumWidth(width)
        grid.addWidget(self.cancelButton, i, 1)

        self.setGeometry(100, 100, 1000, 30 * i)
        self.setLayout(grid)


class LogFileContentDialog(PBDialog):
    """Create a window for showing the logFile content"""

    title = "Log File Content"

    def __init__(self, parent=None):
        """Instantiate class and create its widgets

        Parameter:
            parent: the parent widget
        """
        PBDialog.__init__(self, parent)
        self.textEdit = None
        self.closeButton = None
        self.clearButton = None
        self.initUI()

    def clearLog(self):
        """Ask confirmation, then eventually clear
        the content of the log file
        """
        if askYesNo("Are you sure you want to clear the log file?"):
            try:
                open(pbConfig.params["logFileName"], "w").close()
            except IOError:
                pBGUILogger.exception("Impossible to clear log file!")
            else:
                infoMessage("Log file cleared.")
                self.close()

    def initUI(self):
        """Create window layout and buttons,
        read log file content and print it in the `QPlainTextEdit`
        """
        self.setWindowTitle(self.title)

        grid = QVBoxLayout()
        grid.setSpacing(1)

        grid.addWidget(PBLabel("Reading %s" % pbConfig.params["logFileName"]))
        try:
            with open(pbConfig.params["logFileName"]) as r:
                text = r.read()
        except IOError:
            text = "Impossible to read log file!"
            pBLogger.exception(text)
        self.textEdit = QPlainTextEdit(text)
        self.textEdit.setReadOnly(True)
        grid.addWidget(self.textEdit)

        self.closeButton = QPushButton("Close", self)
        self.closeButton.setAutoDefault(True)
        self.closeButton.clicked.connect(self.close)
        grid.addWidget(self.closeButton)

        self.clearButton = QPushButton("Clear log file", self)
        self.clearButton.clicked.connect(self.clearLog)
        grid.addWidget(self.clearButton)

        self.setGeometry(100, 100, 800, 800)
        self.setLayout(grid)


class PrintText(PBDialog):
    """Create a window for printing text of command line output"""

    stopped = Signal()
    pbMax = Signal(float)
    pbVal = Signal(float)

    def __init__(
        self, parent=None, title="", progressBar=True, noStopButton=False, message=None
    ):
        """Constructor.
        Set some properties and create the GUI of the dialog

        Parameters:
            parent: the parent widget
            title: the window title. If an empty string,
                "Redirect print" will be used
            progressBar: True (default) if the widget must have a progress bar
            noStopButton (default False): True if the widget must have
                a "stop" button to stop the iterations
            message: a text to be inserted as a `PBLabel` in the dialog
        """
        super(PrintText, self).__init__(parent)
        self._wantToClose = False
        self.grid = None
        self.progressBar = None
        self.textEdit = None
        self.closeButton = None
        self.cancelButton = None
        if title != "":
            self.title = title
        else:
            self.title = "Redirect print"
        self.setProgressBar = progressBar
        self.noStopButton = noStopButton
        self.message = message
        self.pbMax.connect(self.progressBarMax)
        self.pbVal.connect(self.progressBarValue)
        self.initUI()

    def keyPressEvent(self, e):
        """Intercept press keys and only exit if escape is pressed
        when closing is enabled

        Parameters:
            e: the `PySide2.QtGui.QKeyEvent`
        """
        if e.key() == Qt.Key_Escape and self._wantToClose:
            self.close()

    def closeEvent(self, event):
        """Manage the `closeEvent` of the dialog.
        Reject unless `self._wantToClose` is True

        Parameter:
            event: a `QEvent`
        """
        if self._wantToClose:
            super(PrintText, self).closeEvent(event)
        else:
            event.ignore()

    def initUI(self):
        """Create the main `QTextEdit`, the buttons and the `QProgressBar`"""
        self.setWindowTitle(self.title)

        grid = QGridLayout()
        self.grid = grid
        grid.setSpacing(1)

        if self.message is not None and self.message.strip() != "":
            grid.addWidget(PBLabel("%s" % self.message))

        self.textEdit = QTextEdit()
        grid.addWidget(self.textEdit)

        if self.setProgressBar:
            self.progressBar = QProgressBar(self)
            grid.addWidget(self.progressBar)

        # cancel button...should learn how to connect it with a thread kill
        if self.noStopButton is not True:
            self.cancelButton = QPushButton("Stop", self)
            self.cancelButton.clicked.connect(self.stopExec)
            self.cancelButton.setAutoDefault(True)
            grid.addWidget(self.cancelButton)
        self.closeButton = QPushButton("Close", self)
        self.closeButton.setDisabled(True)
        grid.addWidget(self.closeButton)

        self.setGeometry(100, 100, 600, 600)
        self.setLayout(grid)
        self.centerWindow()

    def appendText(self, text):
        """Add the given text to the end of the `self.textEdit` content.

        Parameter:
            text: the string to be appended
        """
        self.textEdit.moveCursor(QTextCursor.End)
        self.textEdit.insertPlainText(text)

    def progressBarMin(self, minimum):
        """Set the minimum value for the progress bar

        Parameter:
            minimum (int or float): the value
        """
        if self.setProgressBar:
            self.progressBar.setMinimum(minimum)

    def progressBarMax(self, maximum):
        """Set the maximum value for the progress bar

        Parameter:
            maximum (int or float): the value
        """
        if self.setProgressBar:
            self.progressBar.setMaximum(maximum)

    def progressBarValue(self, value):
        """Set the current value for the progress bar

        Parameter:
            value (int or float): the value
        """
        if self.setProgressBar:
            self.progressBar.setValue(value)

    def stopExec(self):
        """Stop the iterations through the `stopped` Signal
        and disable the `cancelButton`
        """
        self.cancelButton.setDisabled(True)
        self.stopped.emit()

    def enableClose(self):
        """Enable the close button and set `self._wantToClose` to True"""
        self._wantToClose = True
        self.closeButton.setEnabled(True)
        try:
            self.cancelButton.setEnabled(False)
        except Exception:
            pass


class AdvancedImportDialog(PBDialog):
    """create a window for the advanced import"""

    def __init__(self, parent=None):
        """Simple extension of `PBDialog.__init__`"""
        super(AdvancedImportDialog, self).__init__(parent)
        self.result = False
        self.grid = None
        self.searchStr = None
        self.comboMethod = None
        self.acceptButton = None
        self.cancelButton = None
        self.initUI()

    def onCancel(self):
        """Reject the output (set self.result to False and close)"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the output (set self.result to True and close)"""
        self.result = True
        self.close()

    def initUI(self):
        """Create and fill the `QGridLayout`"""
        self.setWindowTitle("Advanced import")

        grid = QGridLayout()
        self.grid = grid
        grid.setSpacing(1)

        ##search
        grid.addWidget(PBLabel("Search string: "), 0, 0)
        self.searchStr = QLineEdit("")
        grid.addWidget(self.searchStr, 0, 1)

        grid.addWidget(PBLabel("Select method: "), 1, 0)
        self.comboMethod = PBComboBox(
            self,
            ["INSPIRE-HEP", "ADS-NASA", "arXiv", "DOI", "ISBN"],
            current="INSPIRE-HEP",
        )
        grid.addWidget(self.comboMethod, 1, 1)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        grid.addWidget(self.acceptButton, 2, 0)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        grid.addWidget(self.cancelButton, 2, 1)

        self.setGeometry(100, 100, 400, 100)
        self.setLayout(grid)
        self.searchStr.setFocus()
        self.centerWindow()


class AdvancedImportSelect(ObjListWindow):
    """create a window for the advanced import"""

    def __init__(self, bibs={}, parent=None):
        """Set some properties and call `initUI`

        Parameters:
            bibs: a dictionary containing the imported bibtex entries.
                Each element should be a dictionary containing at least
                a "bibpars" item, a dictionary with at least the keys
                ["ID", "title", "author", "eprint", "doi"],
                and a boolean "exist" item.
            parent: the parent widget
        """
        self.bibs = bibs
        super(AdvancedImportSelect, self).__init__(parent, gridLayout=True)
        self.checkBoxes = []
        self.result = False
        self.askCats = None
        self.tableModel = None
        self.askCats = None
        self.acceptButton = None
        self.cancelButton = None
        self.initUI()

    def onCancel(self):
        """Reject the output (set self.result to False and close)"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the output (set self.result to True and close)"""
        self.selected = self.tableModel.selectedElements
        self.result = True
        self.close()

    def keyPressEvent(self, e):
        """Intercept press keys and exit if escape is pressed

        Parameters:
            e: the `PySide2.QtGui.QKeyEvent`
        """
        if e.key() == Qt.Key_Escape:
            self.result = False
            self.close()

    def initUI(self):
        """Create and fill the `QGridLayout`"""
        self.setWindowTitle("Advanced import - results")

        self.currLayout.setSpacing(1)

        self.currLayout.addWidget(
            PBLabel(
                "This is the list of elements found."
                + "\nSelect the ones that you want to import:"
            )
        )

        headers = ["ID", "title", "author", "eprint", "doi"]
        for k in self.bibs.keys():
            try:
                self.bibs[k]["bibpars"]["eprint"] = self.bibs[k]["bibpars"]["arxiv"]
            except KeyError:
                pass
            try:
                self.bibs[k]["bibpars"]["author"] = self.bibs[k]["bibpars"]["authors"]
            except KeyError:
                pass
            for f in headers:
                try:
                    self.bibs[k]["bibpars"][f] = self.bibs[k]["bibpars"][f].replace(
                        "\n", " "
                    )
                except KeyError:
                    pass
        self.tableModel = PBImportedTableModel(self, self.bibs, headers)
        self.addFilterInput("Filter entries", gridPos=(1, 0))
        self.setProxyStuff(0, Qt.AscendingOrder)
        self.finalizeTable(gridPos=(2, 0, 1, 2))

        i = 3
        self.askCats = QCheckBox("Ask categories at the end?", self)
        self.askCats.toggle()
        self.currLayout.addWidget(self.askCats, i, 0)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        self.currLayout.addWidget(self.acceptButton, i + 1, 0)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.currLayout.addWidget(self.cancelButton, i + 2, 0)

    def triggeredContextMenuEvent(self, row, col, event):
        """Does nothing"""
        pass

    def handleItemEntered(self, index):
        """Does nothing"""
        pass

    def cellClick(self, index):
        """Does nothing"""
        pass

    def cellDoubleClick(self, index):
        """Does nothing"""
        pass


class DailyArxivDialog(PBDialog):
    """create a window for the advanced import"""

    def __init__(self, parent=None):
        """Simple extension of `PBDialog.__init__`"""
        super(DailyArxivDialog, self).__init__(parent)
        self.result = False
        self.grid = None
        self.comboSub = None
        self.comboCat = None
        self.acceptButton = None
        self.cancelButton = None
        self.initUI()

    def onCancel(self):
        """Reject the output (set self.result to False and close)"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the output (set self.result to True and close)"""
        self.result = True
        self.close()

    def updateCat(self, category):
        """Replace the current content of the `self.comboSub` combobox
        with the new list of subcategories of the given category

        Parameter:
            category: the name of the category in the
                `physBiblioWeb.webSearch["arxiv"].categories` dictionary
        """
        self.comboSub.clear()
        self.comboSub.addItems(
            ["--"] + physBiblioWeb.webSearch["arxiv"].categories[category]
        )

    def initUI(self):
        """Create and fill the `QGridLayout`"""
        self.setWindowTitle("Browse arxiv daily")

        self.grid = QGridLayout()
        self.grid.setSpacing(1)

        ##combo boxes
        self.grid.addWidget(PBLabel("Select category: "), 0, 0)
        self.comboCat = PBComboBox(
            self, [""] + sorted(physBiblioWeb.webSearch["arxiv"].categories.keys())
        )
        self.comboCat.currentIndexChanged[str].connect(self.updateCat)
        self.grid.addWidget(self.comboCat, 0, 1)

        self.grid.addWidget(PBLabel("Subcategory: "), 1, 0)
        self.comboSub = PBComboBox(self, [""])
        self.grid.addWidget(self.comboSub, 1, 1)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        self.grid.addWidget(self.acceptButton, 2, 0)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.grid.addWidget(self.cancelButton, 2, 1)

        self.setLayout(self.grid)

        self.setGeometry(100, 100, 400, 100)
        self.centerWindow()


class DailyArxivSelect(AdvancedImportSelect):
    """create a window for the advanced import"""

    def __init__(self, bibs={}, parent=None):
        """Set some properties and call `initUI`

        Parameters:
            bibs: a dictionary containing the imported bibtex entries.
                Each element should be a dictionary containing at least
                a "bibpars" item, a dictionary with at least the keys
                ["ID", "title", "author", "eprint", "doi"],
                and a boolean "exist" item.
            parent: the parent widget
        """
        self.tableModel = None
        self.askCats = None
        self.acceptButton = None
        self.cancelButton = None
        self.abstractArea = None
        super(DailyArxivSelect, self).__init__(bibs, parent)

    def initUI(self):
        """Initialize the widget content, with the buttons and labels"""
        self.setWindowTitle("ArXiv daily listing - results")

        self.currLayout.setSpacing(1)

        self.currLayout.addWidget(
            PBLabel(
                "This is the list of elements found.\n"
                + "Select the ones that you want to import:"
            )
        )

        headers = ["eprint", "type", "title", "author", "primaryclass"]
        self.tableModel = PBImportedTableModel(
            self, self.bibs, headers + ["abstract"], idName="eprint"
        )
        self.addFilterInput("Filter entries", gridPos=(1, 0))
        self.setProxyStuff(0, Qt.AscendingOrder)

        self.finalizeTable(gridPos=(2, 0, 1, 2))

        i = 3
        self.askCats = QCheckBox("Ask categories at the end?", self)
        self.askCats.toggle()
        self.currLayout.addWidget(self.askCats, i, 0)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        self.currLayout.addWidget(self.acceptButton, i + 1, 0)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.currLayout.addWidget(self.cancelButton, i + 2, 0)

        self.abstractArea = QTextEdit("Abstract", self)
        self.currLayout.addWidget(self.abstractArea, i + 3, 0, 4, 2)

    def cellClick(self, index):
        """Click action

        Parameter:
            index: a `QModelIndex` instance
        """
        if not index.isValid():
            return
        row = index.row()
        try:
            eprint = str(self.proxyModel.sibling(row, 0, index).data())
        except AttributeError:
            pBLogger.debug("Data not valid", exc_info=True)
            return
        if "already existing" in eprint:
            eprint = eprint.replace(" - already existing", "")
        if hasattr(self, "abstractFormulas") and self.abstractFormulas is not None:
            a = self.abstractFormulas(
                self.parent(),
                self.bibs[eprint]["bibpars"]["abstract"],
                customEditor=self.abstractArea,
                statusMessages=False,
            )
            a.doText()
        else:
            pBLogger.debug(
                "self.abstractFormulas not present in "
                + "DailyArxivSelect. Eprint: %s" % eprint
            )


class ExportForTexDialog(PBDialog):
    """create a window for the "export for tex" function"""

    def __init__(self, parent=None):
        """Simple extension of `PBDialog.__init__`"""
        super(ExportForTexDialog, self).__init__(parent)
        self.numTexFields = 1
        self.bibName = ""
        self.texFileNames = [""]
        self.texNames = []
        self.update = False
        self.remove = False
        self.result = False
        self.texButtons = None
        self.bibButton = None
        self.removeCheck = None
        self.updateCheck = None
        self.addTexButton = None
        self.acceptButton = None
        self.cancelButton = None
        self.grid = QGridLayout()
        self.setLayout(self.grid)
        self.initUI()

    def readForm(self):
        """Read the form content"""
        self.update = self.updateCheck.isChecked()
        self.remove = self.removeCheck.isChecked()
        self.bibName = self.bibButton.text()
        if self.bibName == "Select file":
            self.bibName = ""
        self.texFileNames = []
        for button in self.texButtons:
            txt = button.text()
            if "[" in txt:
                try:
                    txt = ast.literal_eval(txt)
                except ValueError:
                    pBLogger.warning("Invalid text: %s" % s)
            self.texFileNames.append(txt)
        tmp = []
        for fn in self.texFileNames:
            if isinstance(fn, list):
                tmp += fn
            else:
                tmp.append(fn)
        self.texNames = [f for f in tmp if f != "Select file" and f != ""]

    def onAddTex(self):
        """Add a new line for the texs"""
        self.numTexFields += 1
        self.readForm()
        for ix in range(self.numTexFields):
            try:
                a = self.texFileNames[ix]
            except IndexError:
                self.texFileNames.append("")
        self.cleanLayout()
        self.initUI()

    def onCancel(self):
        """Reject the output (set self.result to False and close)"""
        self.result = False
        self.close()

    def onOk(self):
        """Accept the output (set self.result to True and close)"""
        self.result = True
        self.readForm()
        self.close()

    def onAskBib(self):
        """Accept the output (set self.result to True and close)"""
        outFName = askSaveFileName(
            self,
            title="Where do you want to export the entries?",
            filter="Bibtex (*.bib)",
        )
        if outFName != "":
            self.bibButton.setText(outFName)

    def onAskTex(self, ix):
        """Accept the output (set self.result to True and close)"""
        texFile = askFileNames(
            self,
            title="Which is/are the *.tex file(s) you want to compile?",
            filter="Latex (*.tex)",
        )
        if texFile != "" and texFile != []:
            self.texButtons[ix].setText("%s" % texFile)

    def initUI(self):
        """Create and fill the `QGridLayout`"""
        self.setWindowTitle("Export for tex file")
        self.grid.setSpacing(1)

        self.grid.addWidget(PBLabelRight("Bib file name:"), 0, 0, 1, 2)
        self.bibButton = QPushButton(
            "Select file" if self.bibName == "" else self.bibName, self
        )
        self.bibButton.clicked.connect(self.onAskBib)
        self.grid.addWidget(self.bibButton, 0, 2, 1, 2)

        self.texButtons = []
        for ix in range(self.numTexFields):
            self.grid.addWidget(PBLabelRight("Tex file name(s):"), ix + 1, 0, 1, 2)
            self.texButtons.append(
                QPushButton(
                    "Select file"
                    if self.texFileNames[ix] == ""
                    else "%s" % self.texFileNames[ix],
                    self,
                )
            )
            self.texButtons[ix].clicked.connect(lambda s=False, p=ix: self.onAskTex(p))
            self.grid.addWidget(self.texButtons[ix], ix + 1, 2, 1, 2)

        i = 2 + self.numTexFields
        self.addTexButton = QPushButton("Add more tex files", self)
        self.addTexButton.clicked.connect(self.onAddTex)
        self.grid.addWidget(self.addTexButton, i - 1, 2)

        self.removeCheck = QCheckBox("Remove unused bibtexs?", self)
        if self.remove:
            self.removeCheck.setChecked(True)
        self.grid.addWidget(self.removeCheck, i, 0, 1, 2)
        self.updateCheck = QCheckBox("Update existing bibtexs?", self)
        if self.update:
            self.updateCheck.setChecked(True)
        self.grid.addWidget(self.updateCheck, i, 2, 1, 2)

        # OK button
        self.acceptButton = QPushButton("OK", self)
        self.acceptButton.clicked.connect(self.onOk)
        self.grid.addWidget(self.acceptButton, i + 1, 1)

        # cancel button
        self.cancelButton = QPushButton("Cancel", self)
        self.cancelButton.clicked.connect(self.onCancel)
        self.cancelButton.setAutoDefault(True)
        self.grid.addWidget(self.cancelButton, i + 1, 2)

        self.setGeometry(100, 100, 400, 25 * (i + 2))
        self.centerWindow()



"""Module that deals with importing info from the INSPIRE-HEP API.

This file is part of the physbiblio package.
"""
import traceback
import re

try:
    from physbiblio.errors import pBLogger
    from physbiblio.config import pbConfig
    from physbiblio.webimport.webInterf import WebInterf, physBiblioWeb
    from physbiblio.parseAccents import parse_accents_str
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise


class WebSearch(WebInterf):
    """Subclass of WebInterf that can connect
    to INSPIRE-HEP to perform searches
    """

    name = "inspire"
    description = "INSPIRE fetcher"
    url = pbConfig.inspireSearchBase
    urlRecord = pbConfig.inspireRecord
    urlArgs = {
        # "action_search": "Search",
        "sf": "year",
        "so": "a",
        "rg": "250",
        "sc": "0",
        "eb": "B",
        "of": "hx"
        # for bibtex format ---- hb for standard format,
        # for retrieving inspireid
    }

    def __init__(self):
        """Initializes the class variables
        using the WebInterf constructor.

        Define additional specific parameters for the INSPIRE-HEP API.
        """
        WebInterf.__init__(self)
        self.name = "inspire"
        self.description = "INSPIRE fetcher"
        self.url = pbConfig.inspireSearchBase
        self.urlRecord = pbConfig.inspireRecord
        self.urlArgs = {
            # "action_search": "Search",
            "sf": "year",
            "so": "a",
            "rg": "250",
            "sc": "0",
            "eb": "B",
            "of": "hx"
            # for bibtex format ---- hb for standard format,
            # for retrieving inspireid
        }

    def retrieveUrlFirst(self, string):
        """Retrieves the first result
        from the content of the given web page.

        Parameters:
            string: the search string

        Output:
            returns the bibtex string obtained from the API
        """
        self.urlArgs["p"] = string.replace(" ", "+")
        url = self.createUrl()
        pBLogger.info("Search '%s' -> %s" % (string, url))
        text = self.textFromUrl(url)
        try:
            i1 = text.find("<pre>")
            i2 = text.find("</pre>")
            if i1 > 0 and i2 > 0:
                bibtex = text[i1 + 5 : i2]
            else:
                bibtex = ""
            return parse_accents_str(bibtex)
        except Exception:
            pBLogger.exception("Impossible to get results")
            return ""

    def retrieveUrlAll(self, string):
        """Retrieves all the result
        from the content of the given web page.

        Parameters:
            string: the search string

        Output:
            returns the bibtex string obtained from the API
        """
        self.urlArgs["p"] = string.replace(" ", "+")
        url = self.createUrl()
        pBLogger.info("Search '%s' -> %s" % (string, url))
        text = self.textFromUrl(url)
        try:
            i1 = text.find("<pre>")
            i2 = text.rfind("</pre>")
            if i1 > 0 and i2 > 0:
                bibtex = text[i1 + 5 : i2]
            else:
                bibtex = ""
            return parse_accents_str(bibtex.replace("<pre>", "").replace("</pre>", ""))
        except Exception:
            pBLogger.exception("Impossible to get results")
            return ""

    def retrieveInspireID(self, string, number=None):
        """Read the fetched content for a given entry
        to obtain its INSPIRE-HEP ID

        Parameters:
            string: the search string
            number (optional): the integer corresponding
                to the desired entry in the list,
                if more than one is present
        """
        i = 0
        self.urlArgs["p"] = string.replace(" ", "+")
        self.urlArgs["of"] = "hb"  # do not ask bibtex, but standard
        url = self.createUrl()
        self.urlArgs["of"] = "hx"  # restore
        pBLogger.info("Search ID of %s -> %s" % (string, url))
        text = self.textFromUrl(url)
        if text is None:
            pBLogger.warning("An error occurred. Empty text obtained")
            return ""
        try:
            searchID = re.compile(
                'titlelink(.*)?(http|https)://inspirehep.net/record/([0-9]*)?">'
            )
            inspireID = ""
            for q in searchID.finditer(text):
                if len(q.group()) > 0:
                    if number is None or i == number:
                        inspireID = q.group(3)
                        break
                    else:
                        i += 1
            pBLogger.info("Found: %s" % inspireID)
            return inspireID
        except Exception:
            pBLogger.exception("Impossible to get results")
            return ""



"""The package that contains the tests for the physbiblio package.

This file is part of the physbiblio package.
"""

from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "test_argParser",
    "test_bibtexWriter",
    "test_config",
    "test_database",
    "test_errors",
    "test_export",
    "test_inspireStats",
    "test_package",
    "test_pdf",
    "test_view",
]



"""Module that deals with importing info from the arXiv API.

Uses feedparser module to read the page content.

This file is part of the physbiblio package.
"""
import sys
import re
import traceback
import feedparser

try:
    from physbiblio.config import pbConfig
    from physbiblio.errors import pBLogger
    from bibtexparser.bibdatabase import BibDatabase
    from physbiblio.webimport.webInterf import WebInterf, physBiblioWeb
    from physbiblio.parseAccents import parse_accents_str
    from physbiblio.bibtexWriter import pbWriter
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise


def getYear(string):
    """Use the arxiv id to compute the year"""
    identif = re.compile("([0-9]{2})([0-9]{2}.[0-9]{4,5}|[0-9]{5})")
    try:
        for t in identif.finditer(string):
            if len(t.group()) > 0:
                a = t.group(1)
                if int(a) > 90:
                    return "19" + a
                else:
                    return "20" + a
    except Exception:
        pBLogger.warning("Error in converting year from '%s'" % string)
        return None


class WebSearch(WebInterf):
    """Subclass of WebInterf that can connect to arxiv.org
    to perform searches
    """

    name = "arXiv"
    description = "arXiv fetcher"
    url = "https://export.arxiv.org/api/query"
    urlRss = "https://export.arxiv.org/rss/"
    urlArgs = {"start": "0"}
    categories = {
        "astro-ph": ["CO", "EP", "GA", "HE", "IM", "SR"],
        "cond-mat": [
            "dis-nn",
            "mes-hall",
            "mtrl-sci",
            "other",
            "quant-gas",
            "soft",
            "stat-mech",
            "str-el",
            "supr-con",
        ],
        "cs": [
            "AI",
            "AR",
            "CC",
            "CE",
            "CG",
            "CL",
            "CR",
            "CV",
            "CY",
            "DB",
            "DC",
            "DL",
            "DM",
            "DS",
            "ET",
            "FL",
            "GL",
            "GR",
            "GT",
            "HC",
            "IR",
            "IT",
            "LG",
            "LO",
            "MA",
            "MM",
            "MS",
            "NA",
            "NE",
            "NI",
            "OH",
            "OS",
            "PF",
            "PL",
            "RO",
            "SC",
            "SD",
            "SE",
            "SI",
            "SY",
        ],
        "econ": ["EM", "GN", "TH"],
        "eess": ["AS", "IV", "SP"],
        "gr-qc": [],
        "hep-ex": [],
        "hep-lat": [],
        "hep-ph": [],
        "hep-th": [],
        "math": [
            "AC",
            "AG",
            "AP",
            "AT",
            "CA",
            "CO",
            "CT",
            "CV",
            "DG",
            "DS",
            "FA",
            "GM",
            "GN",
            "GR",
            "GT",
            "HO",
            "IT",
            "KT",
            "LO",
            "MG",
            "MP",
            "NA",
            "NT",
            "OA",
            "OC",
            "PR",
            "QA",
            "RA",
            "RT",
            "SG",
            "SP",
            "ST",
        ],
        "math-ph": [],
        "nlin": ["AO", "CD", "CG", "PS", "SI"],
        "nucl-ex": [],
        "nucl-th": [],
        "physics": [
            "acc-ph",
            "ao-ph",
            "app-ph",
            "atm-clus",
            "atom-ph",
            "bio-ph",
            "chem-ph",
            "class-ph",
            "comp-ph",
            "data-an",
            "ed-ph",
            "flu-dyn",
            "gen-ph",
            "geo-ph",
            "hist-ph",
            "ins-det",
            "med-ph",
            "optics",
            "plasm-ph",
            "pop-ph",
            "soc-ph",
            "space-ph",
        ],
        "q-bio": ["BM", "CB", "GN", "MN", "NC", "OT", "PE", "QM", "SC", "TO"],
        "q-fin": ["CP", "EC", "GN", "MF", "PM", "PR", "RM", "ST", "TR"],
        "quant-ph": [],
        "stat": ["AP", "CO", "ME", "ML", "OT", "TH"],
    }

    def __init__(self):
        """Initializes the class variables using
        the WebInterf constructor.

        Define additional specific parameters for the arxiv.org API.
        """
        WebInterf.__init__(self)
        self.name = "arXiv"
        self.description = "arXiv fetcher"
        self.url = "https://export.arxiv.org/api/query"
        self.urlRss = "https://export.arxiv.org/rss/"
        self.urlArgs = {"start": "0"}

    def getYear(self, string):
        """Use the arxiv id to compute the year"""
        return getYear(string)

    def retrieveUrlFirst(self, string, searchType="all", **kwargs):
        """Retrieves the first result from the content
        of the given web page.
        The function calls arxivRetriever which will do the job.

        Parameters:
            string: the search string
            searchType: the search method in arxiv API (default 'all')

        Output:
            returns the bibtex string built from arxivRetriever
        """
        return self.arxivRetriever(
            string, searchType, additionalArgs={"max_results": "1"}, **kwargs
        )

    def retrieveUrlAll(self, string, searchType="all", **kwargs):
        """Retrieves all the results from the content
        of the given web page.
        The function calls arxivRetriever which will do the job.

        Parameters:
            string: the search string
            searchType: the search method in arxiv API (default 'all')

        Output:
            returns the bibtex string built from arxivRetriever
        """
        return self.arxivRetriever(
            string,
            searchType,
            additionalArgs={"max_results": pbConfig.params["maxExternalAPIResults"]},
            **kwargs
        )

    def arxivRetriever(
        self, string, searchType="all", additionalArgs=None, fullDict=False
    ):
        """Reads the feed content got from arxiv into a dictionary,
        used to return a bibtex.

        Parameters:
            string: the search string
            searchType: the search method in arxiv API (default 'all').
            The possible values are:
                ti->    Title
                au    ->    Author
                abs    ->    Abstract
                co    ->    Comment
                jr    ->    Journal Reference
                cat    ->    Subject Category
                rn    ->    Report Number
                id    ->    Id (use id_list instead)
                all    ->    All of the above
            additionalArgs: a dictionary of additional arguments
                that can be passed to self.urlArgs (default None)
            fullDict (logical): return the bibtex dictionary in addition
                to the bibtex text (default False)

        Output:
            the bibtex text
            (optional, depending on fullDict): the bibtex Dictionary
        """
        if additionalArgs:
            for k, v in additionalArgs.items():
                self.urlArgs[k] = v
        self.urlArgs["search_query"] = searchType + ":" + string
        url = self.createUrl()
        pBLogger.info("Search '%s:%s' -> %s" % (searchType, string, url))
        text = parse_accents_str(self.textFromUrl(url))
        try:
            data = feedparser.parse(text)
            db = BibDatabase()
            db.entries = []
            dictionaries = []
            for entry in data["entries"]:
                dictionary = {}
                idArx = (
                    entry["id"]
                    .replace("http://arxiv.org/abs/", "")
                    .replace("https://arxiv.org/abs/", "")
                )
                pos = idArx.find("v")
                if pos >= 0:
                    idArx = idArx[0:pos]
                dictionary["ENTRYTYPE"] = "article"
                dictionary["ID"] = idArx
                dictionary["archiveprefix"] = "arXiv"
                dictionary["title"] = entry["title"]
                dictionary["arxiv"] = idArx
                try:
                    dictionary["doi"] = entry["arxiv_doi"]
                except KeyError as e:
                    pBLogger.debug("KeyError: %s" % e)
                dictionary["abstract"] = entry["summary"].replace("\n", " ")
                dictionary["authors"] = " and ".join(
                    [au["name"] for au in entry["authors"]]
                )
                dictionary["primaryclass"] = entry["arxiv_primary_category"]["term"]
                year = self.getYear(dictionary["arxiv"])
                if year is not None:
                    dictionary["year"] = year
                db.entries.append(dictionary)
                dictionaries.append(dictionary)
            if fullDict:
                dictionary = dictionaries[0]
                for d in dictionaries:
                    if string in d["arxiv"]:
                        dictionary = d
                return pbWriter.write(db), dictionary
            else:
                return pbWriter.write(db)
        except Exception:  # intercept all other possible errors
            pBLogger.exception("Impossible to get results")
            if fullDict:
                return "", {}
            else:
                return ""

    def arxivDaily(self, category):
        """Read daily RSS feed for a given category

        Parameter:
            category: the selected category (see `self.categories)
        """
        if "." in category:
            main, sub = category.split(".")
        else:
            main = category
            sub = ""
        url = self.urlRss
        if main not in self.categories.keys():
            pBLogger.warning("Main category not found: %s" % main)
            return False
        else:
            url += main
        if sub != "" and sub not in self.categories[main]:
            pBLogger.warning("Sub category not found: %s" % sub)
            return False
        elif sub != "" and sub in self.categories[main]:
            url += "." + sub
        pBLogger.info(url)
        text = self.textFromUrl(url)
        if text is None:
            pBLogger.warning("Url is empty!")
            return False
        author = re.compile("(>|&gt;)([^/]*)(</a>|&lt;/a&gt;)")
        additionalInfo = re.compile(
            " \(arXiv:([0-9\.v]*) \[([\-\.a-zA-Z]*)\]([ A-Z]*)\)"
        )
        if sys.version_info[0] < 3:
            text = text.decode("utf-8")
        try:
            data = feedparser.parse(parse_accents_str(text))
            entries = []
            for element in data.entries:
                tmp = {}
                tmp["eprint"] = element["id"].split("/")[-1]
                tmp["abstract"] = (
                    element["summary"]
                    .replace("\n", " ")
                    .replace("<p>", "")
                    .replace("</p>", "")
                )
                tmp["authors"] = [
                    m.group(2)
                    for m in author.finditer(element["authors"][0]["name"])
                    if m != ""
                ]
                tmp["author"] = (
                    " and ".join(tmp["authors"])
                    if len(tmp["authors"]) < pbConfig.params["maxAuthorNames"]
                    else " and ".join(
                        tmp["authors"][0 : pbConfig.params["maxAuthorNames"]]
                        + ["others"]
                    )
                )
                tmp["replacement"] = "UPDATED" in element["title"]
                tmp["primaryclass"] = [
                    m.group(2)
                    for m in additionalInfo.finditer(element["title"])
                    if m != ""
                ][0]
                tmp["cross"] = (
                    "CROSS LISTED" in element["title"]
                    or category.lower() not in tmp["primaryclass"].lower()
                )
                tmp["version"] = [
                    m.group(1)
                    for m in additionalInfo.finditer(element["title"])
                    if m != ""
                ][0]
                parenthesis = [
                    m.group()
                    for m in additionalInfo.finditer(element["title"])
                    if m != ""
                ][0]
                tmp["title"] = element["title"].replace(parenthesis, "")
                entries.append(tmp)
            return entries
        except Exception:
            pBLogger.error("Cannot parse arxiv RSS feed:\n%s" % text, exc_info=True)
            return False



"""Module that manages the actions on the database (and few more).

This file is part of the physbiblio package.
"""
import sys
import traceback
import sqlite3
from sqlite3 import (
    DatabaseError,
    IntegrityError,
    InterfaceError,
    OperationalError,
    ProgrammingError,
)
import os
import ast

try:
    import physbiblio.tablesDef
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise

encoding_default = "iso-8859-15"


class PhysBiblioDBCore:
    """Contains most of the basic functions on the database.
    Will be subclassed to do everything else.
    """

    def __init__(self, dbname, logger, noOpen=False, info=True):
        """Initialize database class (column names, descriptions)
        and opens the database.

        Parameters:
            dbname: the name of the database to be opened
            logger: the logger used for messages
            noOpen (boolean, default False): open the database or not
            info (boolean, default True): show some output when opening DB
        """
        # structure of the tables
        self.tableFields = physbiblio.tablesDef.tableFields
        self.descriptions = physbiblio.tablesDef.fieldsDescriptions
        # names of the columns
        self.tableCols = {}
        for q in self.tableFields.keys():
            self.tableCols[q] = [a[0] for a in self.tableFields[q]]

        self.dbChanged = False
        self.conn = None
        self.curs = None
        self.onIsLocked = None
        self.dbname = dbname
        self.logger = logger
        db_is_new = not os.path.exists(self.dbname)

        if not noOpen:
            self.openDB(info=info)
            if db_is_new or self.checkExistingTables():
                self.logger.info(
                    "-------New database or missing tables.\nCreating them!\n\n"
                )
                self.createTables()
            self.checkDatabaseUpdates()

        self.lastFetched = None
        self.catsHier = None

        self.loadSubClasses()

    def openDB(self, info=True):
        """Open the database and creates the self.conn (connection)
        and self.curs (cursor) objects.

        Parameters:
            info (boolean, default True): show some output when opening DB

        Output:
            True
        """
        if info:
            self.logger.info("Opening database: %s" % self.dbname)
        else:
            self.logger.debug("Opening database: %s" % self.dbname)
        self.conn = sqlite3.connect(self.dbname, check_same_thread=False)
        self.conn.row_factory = sqlite3.Row
        self.curs = self.conn.cursor()
        self.loadSubClasses()
        return True

    def reOpenDB(self):
        """Not defined at this stage. Present in subclass PhysBiblioDB"""
        pass

    def closeDB(self, info=True):
        """Close the database.

        Parameters:
            info (boolean, default True): show some output when opening DB
        """
        if info:
            self.logger.info("Closing database...")
        else:
            self.logger.debug("Closing database...")
        self.conn.close()
        return True

    def sendDBIsLocked(self):
        """If the database is open in another instance
        and no writing operation are allowed,
        try to emit a signal to the main GUI
        """
        if self.onIsLocked is not None:
            try:
                self.onIsLocked.emit()
            except AttributeError:
                self.logger.exception("Invalid `self.onIsLocked`!")
                return False
            else:
                return True
        return False

    def checkUncommitted(self):
        """Check if there are uncommitted changes.

        Output:
            True/False
        """
        if sys.version_info[0] == 3 and sys.version_info[1] > 2:
            return self.conn.in_transaction
        else:
            return self.dbChanged

    def commit(self, verbose=True):
        """Commit the changes.

        Parameters:
            verbose (boolean, default True):
                show some output when opening DB

        Output:
            True if successfull, False if an exception occurred
        """
        try:
            self.conn.commit()
        except Exception:
            self.logger.exception("Impossible to commit!")
            return False
        else:
            self.dbChanged = False
            if verbose:
                self.logger.info("Database saved.")
            return True

    def undo(self, verbose=True):
        """Undo the uncommitted changes
        and roll back to the last commit.

        Parameters:
            verbose (boolean, default True):
                show some output when opening DB

        Output:
            True if successfull, False if an exception occurred
        """
        try:
            self.conn.rollback()
        except Exception:
            self.logger.exception("Impossible to rollback!")
            return False
        else:
            self.dbChanged = False
            if verbose:
                self.logger.info("Rolled back to last commit.")
            return True

    def connExec(self, query, data=None):
        """Execute connection.

        Parameters:
            query (string): the query to be executed
            data (dictionary or list):
                the values of the parameters in the query

        Output:
            True if successfull, False if an exception occurred
        """
        try:
            if data:
                self.conn.execute(query, data)
            else:
                self.conn.execute(query)
        except OperationalError as err:
            if str(err) == "database is locked":
                if not self.sendDBIsLocked():
                    self.logger.error(
                        "OperationalError: the database is already open "
                        + "in another instance of the application\n"
                        + "query failed: %s" % query
                    )
            else:
                self.logger.exception("Connection error: %s\nquery: %s" % (err, query))
            return False
        except IntegrityError as err:
            self.logger.exception(
                "Cannot insert/update: ID exists!\n%s\nquery: %s" % (err, query)
            )
            return False
        except (ProgrammingError, DatabaseError, InterfaceError) as err:
            self.logger.exception("Connection error: %s\nquery: %s" % (err, query))
            return False
        else:
            self.dbChanged = True
            return True

    def cursExec(self, query, data=None):
        """Execute cursor.

        Parameters:
            query (string): the query to be executed
            data (dictionary or list):
                the values of the parameters in the query

        Output:
            True if successfull, False if an exception occurred
        """
        try:
            if data:
                self.curs.execute(query, data)
            else:
                self.curs.execute(query)
        except Exception as err:
            self.logger.exception(
                "Cursor error: %s\n" % err
                + 'The query was: "%s"\n' % query
                + " and the parameters: %s" % (data,)
            )
            return False
        else:
            return True

    def cursor(self):
        """Function wrapper that returns the default cursor"""
        return self.curs

    def loadSubClasses(self):
        """Not defined at this stage. Present in subclass PhysBiblioDB"""
        pass

    def checkExistingTables(
        self,
        wantedTables=[
            "categories",
            "entries",
            "entryCats",
            "entryExps",
            "expCats",
            "experiments",
            "settings",
        ],
    ):
        """Check that all the required tables
        are present in the database
        """
        self.cursExec("SELECT name FROM sqlite_master WHERE type='table';")
        tables = [name[0] for name in self.curs]
        return not all(t in tables for t in wantedTables)

    def createTable(self, q, fields, critical=False):
        """Create the table 'q'

        Parameters:
            q: the table name
            fieldsDict: the list containing the column information
                for the table
        """
        command = "CREATE TABLE %s (\n" % q
        first = True
        for el in fields:
            if first:
                first = False
            else:
                command += ",\n"
            command += " ".join(el)
        command += ");"
        self.logger.info(command + "\n")
        if not self.connExec(command):
            self.logger.critical("Create table %s failed" % q)
            if critical:
                sys.exit(1)
        else:
            if critical:
                self.commit()

    def createTables(self, fieldsDict=None):
        """Create tables for the database
        (and insert the default categories), if it is missing.

        Parameters:
            fieldsDict (default None):
                the structure of the tables (see physbiblio.tablesDef)
        """
        if fieldsDict is None:
            fieldsDict = self.tableFields
        self.cursExec("SELECT name FROM sqlite_master WHERE type='table';")
        existingTables = [name[0] for name in self.curs]
        for q in fieldsDict.keys():
            if q in existingTables:
                continue
            self.createTable(q, fieldsDict[q])
        self.cursExec("select * from categories where idCat = 0 or idCat = 1\n")
        cats = self.curs.fetchall()
        if len(cats) < 2:
            command = (
                "INSERT into categories "
                + "(idCat, name, description, parentCat, ord) values "
                + '(0,"Main","This is the main category. '
                + 'All the other ones are subcategories of this one",0,0), '
                + '(1,"Tags","Use this category to store tags '
                + '(such as: ongoing projects, temporary cats,...)",0,0)\n'
            )
            self.logger.info(command + "\n")
            if not self.connExec(command):
                self.logger.error("Insert main categories failed")
        self.commit()

    def checkDatabaseUpdates(self):
        """Run when new columns are added to the database with respect
        to previous versions of the software
        Check if bibdict column is present in entries table
        """
        entryFields = self.cursExec("PRAGMA table_info(entries);")
        entriesCols = [name[1] for name in self.curs]
        if "bibdict" not in entriesCols:
            if self.connExec("ALTER TABLE entries ADD COLUMN bibdict text;"):
                self.logger.info("New column in table 'entries': 'bibdict' (text).")
                self.commit()
            else:
                self.logger.error("Cannot alter table 'entries'!")
                self.undo()


class PhysBiblioDBSub:
    """Uses PhysBiblioDB instance 'self.mainDB = parent'
    to act on the database.
    All the subcategories of PhysBiblioDB are defined
    starting from this one.
    """

    def __init__(self, parent):
        """Initialize DB class, connecting to
        the main PhysBiblioDB instance (parent).
        """
        self.mainDB = parent

        self.tableFields = self.mainDB.tableFields
        self.tableCols = self.mainDB.tableCols

        self.conn = self.mainDB.conn
        self.curs = self.mainDB.curs
        self.dbname = self.mainDB.dbname

        self.lastFetched = None
        self.catsHier = None

    def sendDBIsLocked(self):
        """Call corresponding PhysBiblioDBCore function"""
        return self.mainDB.sendDBIsLocked()

    def literal_eval(self, string):
        """Wrapper for ast.literal_eval

        Parameters:
            string: the string to evaluate

        Output:
            a string or None
        """
        try:
            if "[" in string and "]" in string:
                return ast.literal_eval(string.strip())
            elif "," in string:
                return ast.literal_eval("[%s]" % string.strip())
            else:
                return string.strip()
        except SyntaxError:
            self.mainDB.logger.warning(
                "Error in literal_eval with string '%s'" % string
            )
            return None

    def closeDB(self):
        """Close the database (using PhysBiblioDB.close)"""
        self.mainDB.closeDB()

    def commit(self):
        """Commit the changes (using PhysBiblioDB.commit)"""
        self.mainDB.commit()

    def connExec(self, query, data=None):
        """Execute connection (see PhysBiblioDB.connExec)"""
        return self.mainDB.connExec(query, data=data)

    def cursExec(self, query, data=None):
        """Execute cursor (see PhysBiblioDB.cursExec)"""
        return self.mainDB.cursExec(query, data=data)

    def cursor(self):
        """Return the cursor"""
        return self.mainDB.cursor()



#!/usr/bin/env python
"""Test file for the physbiblio.pdf module.

This file is part of the physbiblio package.
"""
import sys
import traceback
import os
import six
import shutil

if sys.version_info[0] < 3:
    import unittest2 as unittest
    from mock import patch, call
else:
    import unittest
    from unittest.mock import patch, call

try:
    from physbiblio.setuptests import *
    from physbiblio.config import pbConfig
    from physbiblio.database import pBDB, PhysBiblioDB
    from physbiblio.pdf import pBPDF, LocalPDF
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


pBPDF.pdfDir = os.path.join(pbConfig.dataPath, "testpdf_%s" % today_ymd)


@unittest.skipIf(skipTestsSettings.long, "Long tests")
class TestPdfMethods(unittest.TestCase):
    """Test the methods and functions in the pdf module"""

    def test_init(self):
        """Test the __init__ method"""
        with patch(
            "physbiblio.pdf.LocalPDF.checkFolderExists", autospec=True
        ) as _cf, patch.dict(
            pbConfig.params,
            {"pdfFolder": "/a/b/c", "pdfApplication": "someapp"},
            clear=False,
        ):
            pdf = LocalPDF()
            self.assertEqual(pdf.pdfDir, "/a/b/c")
            self.assertEqual(pdf.pdfApp, "someapp")
            self.assertEqual(pdf.badFNameCharacters, "\\/:*?\"<>|'")
            _cf.assert_called_once_with(pdf)

    def test_checkFolderExists(self):
        """Test the checkFolderExists method"""
        with patch(
            "physbiblio.pdf.LocalPDF.checkFolderExists", autospec=True
        ) as _cf, patch.dict(
            pbConfig.params,
            {"pdfFolder": "/a/b/c", "pdfApplication": "someapp"},
            clear=False,
        ):
            pdf = LocalPDF()
        with patch(
            "os.path.isdir", side_effect=[True, False], autospec=True
        ) as _id, patch("logging.Logger.info") as _if, patch("os.makedirs") as _mk:
            pdf.checkFolderExists()
            _id.assert_called_once_with("/a/b/c")
            _if.assert_not_called()
            _mk.assert_not_called()
            pdf.checkFolderExists()
            _if.assert_called_once_with("PDF folder missing: /a/b/c. Creating it.")
            _mk.assert_called_once_with("/a/b/c")

    def test_fnames(self):
        """Test names of folders and directories"""
        self.assertEqual(
            pBPDF.badFName(r'a\b/c:d*e?f"g<h>i|' + "j'"), "a_b_c_d_e_f_g_h_i_j_"
        )
        self.assertEqual(pBPDF.badFName(True), "")
        self.assertEqual(
            pBPDF.getFileDir(r'a\b/c:d*e?f"g<h>i|' + "j'"),
            os.path.join(pBPDF.pdfDir, "a_b_c_d_e_f_g_h_i_j_"),
        )
        testPaperFolder = pBPDF.getFileDir("abc.def")
        with patch(
            "physbiblio.database.Entries.getField",
            return_value="12345678",
            autospec=True,
        ) as _mock:
            self.assertEqual(
                pBPDF.getFilePath("abc.def", "arxiv"),
                os.path.join(testPaperFolder, "12345678.pdf"),
            )

    def test_manageFiles(self):
        """Test creation, copy and deletion of files and folders"""
        emptyPdfName = os.path.join(pBPDF.pdfDir, "tests_%s.pdf" % today_ymd)
        pBPDF.createFolder("abc.def")
        self.assertTrue(os.path.exists(pBPDF.getFileDir("abc.def")))
        pBPDF.renameFolder("abc.def", "abc.fed")
        self.assertFalse(os.path.exists(pBPDF.getFileDir("abc.def")))
        self.assertTrue(os.path.exists(pBPDF.getFileDir("abc.fed")))
        open(emptyPdfName, "a").close()
        with patch(
            "physbiblio.database.Entries.getField",
            return_value="12345678",
            autospec=True,
        ) as _mock:
            pBPDF.copyNewFile("abc.fed", emptyPdfName, "arxiv")
            pBPDF.copyNewFile("abc.fed", emptyPdfName, customName="empty.pdf")
            self.assertTrue(os.path.exists(pBPDF.getFilePath("abc.fed", "arxiv")))
            os.remove(emptyPdfName)
            self.assertFalse(os.path.exists(emptyPdfName))
            pBPDF.copyToDir(pBPDF.pdfDir, "abc.fed", "arxiv")
            self.assertTrue(os.path.exists(os.path.join(pBPDF.pdfDir, "12345678.pdf")))
            pBPDF.mergePDFFolders("abc.fed", "cba.fed")
            self.assertTrue(os.path.exists(pBPDF.getFilePath("cba.fed", "arxiv")))
            self.assertTrue(pBPDF.removeFile("abc.fed", "arxiv"))
            self.assertFalse(pBPDF.removeFile("abc.fed", "arxiv"))
            os.remove(os.path.join(pBPDF.pdfDir, "12345678.pdf"))
            shutil.rmtree(pBPDF.getFileDir("abc.fed"))
            shutil.rmtree(pBPDF.getFileDir("cba.fed"))

    @unittest.skipIf(skipTestsSettings.online, "Online tests")
    def test_download(self):
        """Test downloadArxiv"""
        with patch(
            "physbiblio.database.Entries.getField",
            return_value="1806.11344",
            autospec=True,
        ) as _mock:
            pBPDF.createFolder("abc.def")
            with open(pBPDF.getFilePath("abc.def", "arxiv"), "w") as _fe:
                _fe.write("a")
            self.assertTrue(pBPDF.downloadArxiv("abc.def"))
            self.assertTrue(pBPDF.checkFile("abc.def", "arxiv"))
            self.assertEqual(
                pBPDF.getExisting("abc.def", fullPath=False), ["1806.11344.pdf"]
            )
            self.assertEqual(
                pBPDF.getExisting("abc.def", fullPath=True),
                [os.path.join(pBPDF.getFileDir("abc.def"), "1806.11344.pdf")],
            )
            open(os.path.join(pBPDF.getFileDir("abc.def"), "1806.11344"), "w").close()
            self.assertEqual(
                sorted(pBPDF.getExisting("abc.def", fullPath=False)),
                sorted(["1806.11344", "1806.11344.pdf"]),
            )
            self.assertTrue(
                pBPDF.removeFile(
                    "abc.def",
                    "file",
                    os.path.join(pBPDF.getFileDir("abc.def"), "1806.11344"),
                )
            )
            self.assertEqual(
                pBPDF.getExisting("abc.def", fullPath=False), ["1806.11344.pdf"]
            )
            self.assertTrue(pBPDF.removeFile("abc.def", "arxiv"))
            self.assertFalse(pBPDF.checkFile("abc.def", "arxiv"))
        with patch(
            "physbiblio.database.Entries.getField",
            side_effect=["1801.15000", "1801.15000", "", "", None, None],
            autospec=True,
        ) as _mock:
            self.assertFalse(pBPDF.downloadArxiv("abc.def"))
            self.assertFalse(pBPDF.downloadArxiv("abc.def"))
            self.assertFalse(pBPDF.downloadArxiv("abc.def"))
        shutil.rmtree(pBPDF.getFileDir("abc.def"))

    def test_removeSpare(self):
        """Test finding spare folders"""
        with patch(
            "physbiblio.database.Entries.fetchCursor",
            return_value=[{"bibkey": "abc"}, {"bibkey": "def"}],
            autospec=True,
        ) as _mock:
            for q in ["abc", "def", "ghi"]:
                pBPDF.createFolder(q)
                self.assertTrue(os.path.exists(pBPDF.getFileDir(q)))
            pBPDF.removeSparePDFFolders()
            for q in ["abc", "def"]:
                self.assertTrue(os.path.exists(pBPDF.getFileDir(q)))
            self.assertFalse(os.path.exists(pBPDF.getFileDir("ghi")))
        shutil.rmtree(pBPDF.pdfDir)

    def test_numberOfFiles(self):
        """test numberOfFiles"""
        self.assertEqual(pBPDF.numberOfFiles("/surely/non/existent/folder"), 0)
        if sys.version_info[0] < 3:
            with patch(
                "os.listdir", side_effect=[["a", "b"], ["c", "d", "e"]], autospec=True
            ) as _ld, patch(
                "os.path.isfile",
                side_effect=[True, False, True, True, False],
                autospec=True,
            ) as _if, patch(
                "os.path.isdir", side_effect=[True, False], autospec=True
            ) as _id:
                self.assertEqual(pBPDF.numberOfFiles("f"), 3)
                _ld.assert_has_calls([call("f"), call("f/b")])
                _if.assert_has_calls(
                    [
                        call("f/a"),
                        call("f/b"),
                        call("f/b/c"),
                        call("f/b/d"),
                        call("f/b/e"),
                    ]
                )
                _id.assert_has_calls([call("f/b"), call("f/b/e")])
        else:
            with patch(
                "os.walk",
                return_value=[["f", ["b"], ["a", "b/c", "b/d", "b/e"]]],
                autospec=True,
            ) as _wa, patch(
                "os.path.isfile", side_effect=[True, True, True, False], autospec=True
            ) as _if:
                self.assertEqual(pBPDF.numberOfFiles("f"), 3)
                _wa.assert_called_once_with("f")
                _if.assert_has_calls(
                    [call("f/a"), call("f/b/c"), call("f/b/d"), call("f/b/e")]
                )

    def test_dirSize(self):
        """test dirSize"""
        if six.PY2:
            error_class = OSError
        else:
            error_class = FileNotFoundError
        with patch("logging.Logger.error") as _e, patch("os.makedirs") as _md, patch(
            "os.path.getsize", side_effect=[error_class, 123]
        ) as _gs:
            self.assertEqual(pBPDF.dirSize("/surely/non/existent/folder"), 123)
            _e.assert_called_once_with(
                "PDF folder is missing: /surely/non/existent/folder. Creating it."
            )
            _md.assert_called_once_with("/surely/non/existent/folder")
            _gs.assert_any_call("/surely/non/existent/folder")
        if sys.version_info[0] < 3:
            with patch(
                "os.path.getsize", side_effect=[1, 100, 1, 100, 100], autospec=True
            ) as _gs, patch(
                "os.listdir", side_effect=[["a", "b"], ["c", "d", "e"]], autospec=True
            ) as _ld, patch(
                "os.path.isfile",
                side_effect=[True, False, True, True, False],
                autospec=True,
            ) as _if, patch(
                "os.path.isdir", side_effect=[True, False], autospec=True
            ) as _id:
                self.assertEqual(pBPDF.dirSize("f"), 302)
                _gs.assert_has_calls(
                    [call("f"), call("f/a"), call("f/b"), call("f/b/c"), call("f/b/d")]
                )
                _ld.assert_has_calls([call("f"), call("f/b")])
                _if.assert_has_calls(
                    [
                        call("f/a"),
                        call("f/b"),
                        call("f/b/c"),
                        call("f/b/d"),
                        call("f/b/e"),
                    ]
                )
                _id.assert_has_calls([call("f/b"), call("f/b/e")])
            with patch(
                "os.path.getsize", return_value=100, autospec=True
            ) as _gs, patch(
                "os.listdir", side_effect=[["a", "b"], ["c", "d", "e"]], autospec=True
            ) as _ld, patch(
                "os.path.isfile",
                side_effect=[True, False, True, True, False],
                autospec=True,
            ) as _if, patch(
                "os.path.isdir", side_effect=[True, False], autospec=True
            ) as _id:
                self.assertEqual(pBPDF.dirSize("f", dirs=False), 300)
                _gs.assert_has_calls([call("f/a"), call("f/b/c"), call("f/b/d")])
                _ld.assert_has_calls([call("f"), call("f/b")])
                _if.assert_has_calls(
                    [
                        call("f/a"),
                        call("f/b"),
                        call("f/b/c"),
                        call("f/b/d"),
                        call("f/b/e"),
                    ]
                )
                _id.assert_has_calls([call("f/b"), call("f/b/e")])
        else:
            with patch(
                "os.path.getsize", side_effect=[1, 1, 100, 100, 100], autospec=True
            ) as _gs, patch(
                "os.walk",
                return_value=[["f", ["b"], ["a", "b/c", "b/d", "b/e"]]],
                autospec=True,
            ) as _wa, patch(
                "os.path.isfile", side_effect=[True, True, True, False], autospec=True
            ) as _if:
                self.assertEqual(pBPDF.dirSize("f"), 302)
                _gs.assert_has_calls(
                    [call("f"), call("f/b"), call("f/a"), call("f/b/c"), call("f/b/d")]
                )
                _wa.assert_called_once_with("f")
                _if.assert_has_calls(
                    [call("f/a"), call("f/b/c"), call("f/b/d"), call("f/b/e")]
                )
            with patch(
                "os.path.getsize", side_effect=[100, 100, 100], autospec=True
            ) as _gs, patch(
                "os.walk",
                return_value=[["f", ["b"], ["a", "b/c", "b/d", "b/e"]]],
                autospec=True,
            ) as _wa, patch(
                "os.path.isfile", side_effect=[True, True, True, False], autospec=True
            ) as _if:
                self.assertEqual(pBPDF.dirSize("f", dirs=False), 300)
                _gs.assert_has_calls([call("f/a"), call("f/b/c"), call("f/b/d")])
                _wa.assert_called_once_with("f")
                _if.assert_has_calls(
                    [call("f/a"), call("f/b/c"), call("f/b/d"), call("f/b/e")]
                )

    def test_getSizeWUnits(self):
        """test getSizeWUnits"""
        with patch("logging.Logger.warning") as _w:
            self.assertEqual(pBPDF.getSizeWUnits(2048 ** 2), "4.00MB")
            self.assertEqual(pBPDF.getSizeWUnits(2048 ** 2, units="mb"), "4.00MB")
            self.assertEqual(pBPDF.getSizeWUnits(2048 ** 2, units="kb"), "4096.00KB")
            self.assertEqual(pBPDF.getSizeWUnits(2048 ** 2, units="PB"), "0.00PB")
            self.assertEqual(
                pBPDF.getSizeWUnits(2048 ** 2, units="gb", fmt="%.3f"), "0.004GB"
            )
            _w.assert_not_called()
        with patch("logging.Logger.warning") as _w:
            self.assertEqual(pBPDF.getSizeWUnits(2048 ** 2, units="m"), "4.00MB")
            _w.assert_called_once_with("Invalid units. Changing to 'MB'.")
        with patch("logging.Logger.warning") as _w:
            self.assertEqual(pBPDF.getSizeWUnits("a"), "nan")
            _w.assert_called_once_with("Invalid size. It must be a number!")
        with patch("logging.Logger.warning") as _w:
            self.assertEqual(pBPDF.getSizeWUnits(2048 ** 2, fmt="%"), "4.00MB")
            _w.assert_called_once_with("Invalid format. Using '%.2f'")


if __name__ == "__main__":
    unittest.main()



"""The package that contains all the GUI modules used by the PhysBiblio.

This file is part of the physbiblio package.
"""
from physbiblio import __author__, __email__, __version__, __version_date__

__all__ = [
    "basicDialogs",
    "bibWindows",
    "catWindows",
    "commonClasses",
    "dialogWindows",
    "errorManager",
    "expWindows",
    "inspireStatsGUI",
    "mainWindow",
    "marks",
    "profilesManager",
    "resourcesPyside2",
    "setuptests",
    "threadElements",
]



#!/usr/bin/env python
"""Script for installing, uploading, doing tests and so on.

This file is part of the physbiblio package.
"""

from setuptools import setup

import physbiblio


def readme():
    with open("README.md") as f:
        return f.read()


setup(
    name="physbiblio",
    version=physbiblio.__version__,
    description="A bibliography manager in Python (using Sqlite and PySide2)",
    long_description_content_type="text/markdown",
    long_description=readme(),
    author=physbiblio.__author__,
    author_email=physbiblio.__email__,
    url="https://github.com/steog88/PhysBiblio",
    license="GPL-3.0",
    keywords=("bibliography", "hep-ph", "high-energy-physics", "bibtex"),
    packages=[
        "physbiblio",
        "physbiblio.gui",
        "physbiblio.webimport",
        "physbiblio.tests",
        "physbiblio.gui.tests",
        "physbiblio.webimport.tests",
    ],
    scripts=["PhysBiblio.exe"],
    package_data={"": ["*.sh", "*.md", "*.png"], "physbiblio.gui": ["images/*.png"]},
    install_requires=[
        "ads",
        "appdirs",
        "argparse",
        "bibtexparser(>=1.1.0)",
        "dictdiffer",
        'faulthandler;python_version<"3"',
        "feedparser",
        'matplotlib(>2.2.0, <3);python_version<"3"',
        'matplotlib(>3.0.0);python_version>"3"',
        "outdated",
        "pylatexenc",
        "pymarc",
        "pyoai",
        "pyparsing(>=2.4.0)",
        "pyside2(>=5.12.2)",
        "requests",
        "six",
        'mock;python_version<"3"',
        'unittest2;python_version<"3"',
    ],
    classifiers=(
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ),
    provides=["physbiblio"],
    data_files=[
        ("physbiblio", ["LICENSE", "CHANGELOG", "physbiblio/gui/images/icon.png"])
    ],
    test_loader="physbiblio.testLoader:PBScanningLoader",
)
